dnl -*- m4 -*-
dnl Local Macros for PCC

dnl ----------------------------------------------------------------------
dnl
dnl  AC_PATH_SDL. Stolen from SDL_image (where it is called `AM_PATH_SDL').
dnl  This macro must be called quite early, because it compiles C.
dnl  All further tests are done in C++.
dnl

# Configure paths for SDL
# Sam Lantinga 9/21/99
# stolen from Manish Singh
# stolen back from Frank Belew
# stolen from Manish Singh
# Shamelessly stolen from Owen Taylor

dnl AC_PATH_SDL([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl Test for SDL, and define SDL_CFLAGS and SDL_LIBS
dnl
AC_DEFUN(AC_PATH_SDL,
[dnl 
dnl Get the cflags and libraries from the sdl-config script
dnl
AC_ARG_WITH(sdl-prefix,[  --with-sdl-prefix=PFX   Prefix where SDL is installed (optional)],
            sdl_prefix="$withval", sdl_prefix="")
AC_ARG_WITH(sdl-exec-prefix,[  --with-sdl-exec-prefix=PFX Exec prefix where SDL is installed (optional)],
            sdl_exec_prefix="$withval", sdl_exec_prefix="")
AC_ARG_ENABLE(sdltest, [  --disable-sdltest       Do not try to compile and run a test SDL program],
		    , enable_sdltest=yes)

  if test x$sdl_exec_prefix != x ; then
     sdl_args="$sdl_args --exec-prefix=$sdl_exec_prefix"
     if test x${SDL_CONFIG+set} != xset ; then
        SDL_CONFIG=$sdl_exec_prefix/bin/sdl-config
     fi
  fi
  if test x$sdl_prefix != x ; then
     sdl_args="$sdl_args --prefix=$sdl_prefix"
     if test x${SDL_CONFIG+set} != xset ; then
        SDL_CONFIG=$sdl_prefix/bin/sdl-config
     fi
  fi

  AC_PATH_PROG(SDL_CONFIG, sdl-config, no)
  min_sdl_version=ifelse([$1], ,0.11.0,$1)
  AC_MSG_CHECKING(for SDL - version >= $min_sdl_version)
  no_sdl=""
  if test "$SDL_CONFIG" = "no" ; then
    no_sdl=yes
  else
    SDL_CFLAGS=`$SDL_CONFIG $sdlconf_args --cflags`
    SDL_LIBS=`$SDL_CONFIG $sdlconf_args --libs`

    sdl_major_version=`$SDL_CONFIG $sdl_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    sdl_minor_version=`$SDL_CONFIG $sdl_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    sdl_micro_version=`$SDL_CONFIG $sdl_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
    if test "x$enable_sdltest" = "xyes" ; then
      ac_save_CFLAGS="$CFLAGS"
      ac_save_LIBS="$LIBS"
      CFLAGS="$CFLAGS $SDL_CFLAGS"
      LIBS="$LIBS $SDL_LIBS"
dnl
dnl Now check if the installed SDL is sufficiently new. (Also sanity
dnl checks the results of sdl-config to some extent
dnl
      rm -f conf.sdltest
      AC_TRY_RUN([
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "SDL.h"

char*
my_strdup (char *str)
{
  char *new_str;
  
  if (str)
    {
      new_str = (char *)malloc ((strlen (str) + 1) * sizeof(char));
      strcpy (new_str, str);
    }
  else
    new_str = NULL;
  
  return new_str;
}

int main (int argc, char *argv[])
{
  int major, minor, micro;
  char *tmp_version;

  /* This hangs on some systems (?)
  system ("touch conf.sdltest");
  */
  { FILE *fp = fopen("conf.sdltest", "a"); if ( fp ) fclose(fp); }

  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = my_strdup("$min_sdl_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string\n", "$min_sdl_version");
     exit(1);
   }

   if (($sdl_major_version > major) ||
      (($sdl_major_version == major) && ($sdl_minor_version > minor)) ||
      (($sdl_major_version == major) && ($sdl_minor_version == minor) && ($sdl_micro_version >= micro)))
    {
      return 0;
    }
  else
    {
      printf("\n*** 'sdl-config --version' returned %d.%d.%d, but the minimum version\n", $sdl_major_version, $sdl_minor_version, $sdl_micro_version);
      printf("*** of SDL required is %d.%d.%d. If sdl-config is correct, then it is\n", major, minor, micro);
      printf("*** best to upgrade to the required version.\n");
      printf("*** If sdl-config was wrong, set the environment variable SDL_CONFIG\n");
      printf("*** to point to the correct copy of sdl-config, and remove the file\n");
      printf("*** config.cache before re-running configure\n");
      return 1;
    }
}

],, no_sdl=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CFLAGS="$ac_save_CFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi
  if test "x$no_sdl" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$SDL_CONFIG" = "no" ; then
       echo "*** The sdl-config script installed by SDL could not be found"
       echo "*** If SDL was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the SDL_CONFIG environment variable to the"
       echo "*** full path to sdl-config."
     else
       if test -f conf.sdltest ; then
        :
       else
          echo "*** Could not run SDL test program, checking why..."
          CFLAGS="$CFLAGS $SDL_CFLAGS"
          LIBS="$LIBS $SDL_LIBS"
          AC_TRY_LINK([
#include <stdio.h>
#include "SDL.h"
],      [ return 0; ],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding SDL or finding the wrong"
          echo "*** version of SDL. If it is not finding SDL, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
	  echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means SDL was incorrectly installed"
          echo "*** or that you have moved SDL since it was installed. In the latter case, you"
          echo "*** may want to edit the sdl-config script: $SDL_CONFIG" ])
          CFLAGS="$ac_save_CFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi
     SDL_CFLAGS=""
     SDL_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(SDL_CFLAGS)
  AC_SUBST(SDL_LIBS)
  rm -f conf.sdltest
])

dnl ----------------------------------------------------------------------
dnl
dnl  AC_FUNC_STREAM_TRAITS
dnl  -> define HAVE_STREAM_TRAITS if the `std::iostream::traits'
dnl     works
dnl
AC_DEFUN(AC_FUNC_STREAM_TRAITS, [
  AC_CACHE_CHECK(for working stream traits, ac_cv_have_stream_traits, [
  AC_TRY_COMPILE([#include <iostream>], [
    int i = std::istream::traits::eof();
    return i;], ac_cv_have_stream_traits=yes, ac_cv_have_stream_traits=no)])
  if test $ac_cv_have_stream_traits = yes; then
    AC_DEFINE(HAVE_STREAM_TRAITS)
  fi
])

dnl ----------------------------------------------------------------------
dnl
dnl  AC_FUNC_STREAM_TYPES
dnl  -> define HAVE_STREAM_TYPES if `std::streambuf' contains the
dnl     manifest types (pos_type etc.)
dnl
AC_DEFUN(AC_FUNC_STREAM_TYPES, [
  AC_CACHE_CHECK(for stream manifest types, ac_cv_have_stream_types, [
    AC_TRY_COMPILE([#include <iostream>], [
      std::streambuf::pos_type i = 0;
    ], ac_cv_have_stream_types=yes, ac_cv_have_stream_types=no)])
  if test $ac_cv_have_stream_types = yes; then
    AC_DEFINE(HAVE_STREAM_TYPES)
  fi
])

dnl ----------------------------------------------------------------------
dnl
dnl  AC_FUNC_STREAM_ERRNO
dnl  -> define HAVE_STREAM_ERRNO if iostreams affect errno
dnl
AC_DEFUN(AC_FUNC_STREAM_ERRNO, [
  AC_CACHE_CHECK([whether iostreams set errno], ac_cv_have_stream_errno, [
  AC_TRY_RUN([#include <fstream>
#include <cerrno>
int main() {
    errno = 0;
    std::ifstream f("non-existant-file");
    return errno==0 ? 1 : 0;
}], ac_cv_have_stream_errno=yes, ac_cv_have_stream_errno=no, ac_cv_have_stream_errno=yes)])
  if test $ac_cv_have_stream_errno = yes; then
    AC_DEFINE(HAVE_STREAM_ERRNO)
  fi
])

dnl ----------------------------------------------------------------------
dnl
dnl  AC_FUNC_COPY_N
dnl  -> define HAVE_COPY_N if <algorithm> has copy_n
dnl
AC_DEFUN(AC_FUNC_COPY_N, [
  AC_CACHE_CHECK([for copy_n], ac_cv_have_copy_n, [
  AC_TRY_COMPILE([#include <algorithm>],dnl
    [char *in, *out; std::copy_n(in, 20, out); return 0;],dnl
    ac_cv_have_copy_n=yes, ac_cv_have_copy_n=no)])
  if test $ac_cv_have_copy_n = yes; then
    AC_DEFINE(HAVE_COPY_N)
  fi
])

AC_DEFUN(AC_CHECK_FUNCALL, [
  AC_CACHE_CHECK([for $1], ac_cv_have_$1, [
  AC_TRY_COMPILE([#include $2],[$3],dnl
    ac_cv_have_$1=yes, ac_cv_have_$1=no)])
  if test $ac_cv_have_$1 = yes; then
    AC_DEFINE(HAVE[_]translit($1,a-z,A-Z))
  fi
])
