/**
  *  \file arch/generic.h
  */
#ifndef PCC_V2_ARCH_GENERIC_H
#define PCC_V2_ARCH_GENERIC_H

#include "cpluslib/string.h"

class StandardCommandLineParser;
void enableStdioConsole();

struct GenericPlatform {
    /** Program initialisation. A program that uses potentially uses
        graphics mode must call init() after SDL_Init() resp. 
        after determining that graphics will not be used. Rationale:
        graphics programs must be setuid for SVGAlib. When running in
        X11 or text mode, we must drop root privileges ourselves.

        Programs that run in text mode exclusively need not call
        this function (if user makes them setuid, it's his fault...). */
    static inline void init(bool /*graphics*/) { }

    /** Initialize options. Call this before parameter parsing. */
    static inline void initOptions(StandardCommandLineParser&) { }

    /** Show output. Call this before exiting when the GUI was not
        run, from a GUI program. */
    static inline void initOutput() { enableStdioConsole(); }
    static inline void showOutput() { }

    /** Get data directory (planets root). Unimplemented, platform
        code MUST override it. */
    static string_t getDataDirectory();

    /** Get resource directory. Unimplemented, platform code MUST
        override it. */
    static string_t getResourceDirectory();

    /** Get user home directory. Unimplemented, platform code MUST
        override it. */
    static string_t getUserSettingsDirectory();

    /** Get language code. */
    static string_t getLanguageCode() { return string_t(); }

    static const char PATH_SEPARATOR = '/';
};

#endif
