/**
  *  \file arch/unix/platform.cc
  *  \brief Platform-specific things for Unix
  */

#include <cstdlib>
#include <sys/stat.h>
#include <unistd.h>
#include "arch/platform.h"      // not arch/unix/platform.h, we need PCC2_xxxDIR
#include "defs.h"
#include "version.h"

void
Platform::init(bool /*graphics*/)
{
    /* when running as root, (attempt to) drop privileges. */
    if (geteuid() == 0)
        setuid(getuid());
}

/****************************** Directories ******************************/

/** true iff directory fn exists. */
static bool
exists(const char* fn)
{
    struct stat st;
    return stat(fn, &st) == 0 && S_ISDIR(st.st_mode);
}

/** Get data directory, helper. Data directory is either
    $pdatadir or $pdatadir/version. */
static string_t
getDataDirectoryH()
{
    if (exists(PCC2_DATADIR "/" PCC2_VERSION))
        return string_t(PCC2_DATADIR "/" PCC2_VERSION);
    else
        return string_t(PCC2_DATADIR);
}

/** Get data directory (planets root). */
string_t
Platform::getDataDirectory()
{
#ifdef HAVE_INPLACE_ENABLED
    if (exists(PCC2_SRCDIR "/specs"))
        return string_t(PCC2_SRCDIR "/specs");
#endif
    return getDataDirectoryH();
}

/** Get resource directory. */
string_t
Platform::getResourceDirectory()
{
#ifdef HAVE_INPLACE_ENABLED
    if (exists(PCC2_SRCDIR "/resource"))
        return string_t(PCC2_SRCDIR "/resource");
#endif
    return getDataDirectoryH() + "/resource";
}

string_t
Platform::getLanguageCode()
{
    if (const char* c = std::getenv("LC_ALL"))
        return c;
    if (const char* c = std::getenv("LC_MESSAGES"))
        return c;
    if (const char* c = std::getenv("LANG"))
        return c;
    return string_t();
}

string_t
Platform::getUserSettingsDirectory()
{
    if (const char* c = std::getenv("HOME"))
        return c;
    else
        return "/";
}
