/**
  *  \file arch/win32-bcc55/platform.cc
  */

#include <windows.h>
#include "arch/win32-bcc55/platform.h"
#include "fileio/dirs.h"

static string_t
getProgDirName()
{
    char buffer[1024];
    int len = GetModuleFileName(NULL, buffer, sizeof(buffer));
    return Directory::getDirName(string_t(buffer, len));
}

string_t
Platform::getDataDirectory()
{
    return Directory::makeFileName(getProgDirName(), "specs");
}

string_t
Platform::getResourceDirectory()
{
    return Directory::makeFileName(getProgDirName(), "resource");
}

string_t
Platform::getLanguageCode()
{
    /* According to Win32.hlp:
       # LOCALE_SABBREVLANGNAME:
       #    Abbreviated name of the language, created by taking the
       #    2-letter language abbreviation from the ISO Standard 639 and
       #    adding a third letter, as appropriate, to indicate the
       #    sublanguage.
       So, when we get `enu', we make `en_u' from it. Under Unix,
       the relevant code would be `en_US', but we only use the part
       before the underscore anyway. */
    
    char buffer[10];
    int n = GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SABBREVLANGNAME,
                          buffer, sizeof buffer);
    if (n <= 1) {
        return string_t();
    } else {
        --n;
        if (n <= 2)
            return string_t(buffer, n);
        else
            return string_t(buffer, 2) + "_" + string_t(buffer+2, n-2);
    }
}
