/**
  *  \file client/actions.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_ACTIONS_H
#define PCC_V2_CLIENT_ACTIONS_H

#include "game/objlist.h"
#include "game/transact.h"

class GPlanet;
class GShip;
class UIEvent;
class GCargoContainer;

void doPlanetFCodeChange(GPlanet& planet);
void doPlanetTransfer(GPlanet& planet, bool unload);
void doPlanetTransferFor(GPlanet& planet, GObjectId chosen);

void doBaseChangeMission(GPlanet& planet);
void doBaseFixRecycle(GPlanet& planet, const UIEvent& pe);

class GStarbaseFixRecycleAction {
    int      num_own, num_foreign;
    GPlanet& the_planet;
 public:
    GStarbaseFixRecycleAction(GPlanet& the_planet);
    bool canFix() const;
    bool canRecycle() const;
    bool setAction(int action, int ship_id);
    void enumerateShipsFor(int action, GObjectList& list);
};

class GStarbaseTechUpgradeAction {
    GPlanet& the_planet;
    GCargoContainer& container;
    int      new_tech[4], min_tech[4];
    int      reserved_sup, reserved_mc, rest;

    void     changeAllocation(int amount);
 public:
    GStarbaseTechUpgradeAction(GPlanet& the_planet, GCargoContainer& container);
    ~GStarbaseTechUpgradeAction();

    bool     isValid(int index, int new_val) const;
    bool     setTech(int index, int new_val);
    bool     isValid() const;
    int      getCost() const;

    void     commit();
    void     cancel();

    int      getTech(int index) const    { return new_tech[index]; }
    int      getMinTech(int index) const { return min_tech[index]; }
    int      getMaxTech(int index) const;

    int      getMissingCash() const { return rest; }
    int      getReservedCash() const { return reserved_mc; }
    int      getReservedSupplies() const { return reserved_sup; }
};

#endif
