/**
  *  \file client/chart.h
  *  \brief Starchart Drawing
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_CHART_H
#define PCC_V2_CLIENT_CHART_H

#include <SDL.h>
#include "cpluslib/smartptr.h"
#include "gfx/gfx.h"
#include "game/coord.h"
#include "game/global.h"

class GGameTurn;

extern const GfxFillPattern IONSTORM_FILL, IONSTORM_DENSE_FILL;

//! Chart Options (what/how to display).
struct GChartOptions {
    int32 show;
    int32 fill;

    /** Starchart Option Bitflags */
    enum {
        co_Ion       = 1,
        co_Mine      = 2,
        co_Ufo       = 4,
        co_Sectors   = 8,
        co_Borders   = 16,
        co_Drawings  = 32,
        co_Selection = 64,
        co_Labels    = 128,
        co_Trails    = 256,

        co_Tristate  = co_Ion + co_Mine + co_Ufo,

        co_Default   = ~0U - co_Trails
    };
};

//! Chart Viewport.
class GChartViewport {
    int x0, y0;
    int xm, ym;

    void drawMovingObject(GfxCanvas* can, GfxParameters& parm, GfxPoint pt,
                          int r, int speed, int heading, int32 f);
    
    void drawAux(Ptr<GfxCanvas> can, GChartOptions& opt, GGameTurn& trn,
                 const GChartBBox& bbox);
    void drawShipSelAndVectors(Ptr<GfxCanvas> can, GGameTurn& trn, int show);
    void drawPlanets(Ptr<GfxCanvas> can, GGameTurn& trn, int show);
    void drawShips(Ptr<GfxCanvas> can, GGameTurn& trn, int show);
    void drawSectors(Ptr<GfxCanvas> can, int show);
    void drawMines(Ptr<GfxCanvas> can, GGameTurn& trn, int fill);
    void drawCross(Ptr<GfxCanvas> can, GfxParameters& parm, GfxPoint pt);
    void drawIons(Ptr<GfxCanvas> can, GGameTurn& trn, int fill);
 public:
    GfxRect view;               /**< Screen positions. */
    GPoint center;              /**< Center coordinate of map. */
    int mult, divi;
    int vec_id;

    void init();
    void draw(Ptr<GfxCanvas> can, GChartOptions& opt, GGameTurn& trn,
              const GChartBBox& bbox);
    int scale(int n) const;
    int scaleClip(int n, int add) const;
    int scaleX(int x) const { return x0 + scale(x); }
    int scaleY(int y) const { return y0 - scale(y); }
    GfxPoint scaleXY(GPoint pt) const { return GfxPoint(scaleX(pt.x), scaleY(pt.y)); }

    int unscale(int n) const;
    int unscaleX(int x) const { return unscale(x - x0); }
    int unscaleY(int y) const { return unscale(y0 - y); }
    GPoint unscaleXY(GfxPoint pt) const { return GPoint(unscaleX(pt.x), unscaleY(pt.y)); }

    int getCenterX() const { return xm; }
    int getCenterY() const { return ym; }
    GfxPoint getCenter() const { return GfxPoint(xm, ym); }

    bool zoomIn(GChartBBox& bbox);
    bool zoomOut(GChartBBox& bbox);
};

GChartOptions& getChartOpts(bool is_scanner, int mult, int divi);

#endif
