/**
  *  \file client/cscreen.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_CSCREEN_H
#define PCC_V2_CLIENT_CSCREEN_H

#include "client/chart.h"
#include "cpluslib/smartptr.h"
#include "game/objtypes.h"
#include "resmgr/resmgr.h"
#include "ui/commands.h"
#include "ui/ui.h"
#include "cpluslib/holder.h"

enum {
//     cm_F1 = cm_FirstUser,
//     cm_F2,
//     cm_F3,
//     cm_F4,
//     cm_F5,
//     cm_F6,
//     cm_F7,
//     cm_F8,
//     cm_F9,
//     cm_F10,

    cm_CS_ToggleSel = cm_ControlScreenFirst,
    cm_CS_Xchg,                    // "X"
    cm_CS_ReturnFromCS,
    cm_CS_ScannerDouble,
    cm_CS_MoveManually,            // "Y"
    cm_CS_ListShips,               // "L"
    cm_CS_GotoCS                   // args are screen/object id
//     cm_CS_Reset,                // "Alt-R"
//     cm_CS_Console,              // "Alt-C"
//     cm_CS_Taskmanager,          // "Alt-Q"
    // cm_EditorUpdated
//     cm_ToggleAuto,              // "Enter"

//     cm_CS_FirstUser = 2000
};

class WControlScreen;

class WTile : public UIWidget {
    GfxPoint size;
    int      update_when;
 public:
    enum Color {
        tc_Static,
        tc_Green,
        tc_Yellow,
        tc_Red,
        tc_White
    };
    enum Theme {
        tt_DarkGray,            ///< Control Screen.
        tt_LightGray,           ///< Selection Window.
        tt_Transparent
    };
    Theme theme;
 protected:
    GfxParameters parm;
    virtual void drawFixed(Ptr<GfxCanvas> can);
    virtual void drawData(Ptr<GfxCanvas> can);
    void setColor(Color c);
    void outText(GfxCanvas* can, int f, int x, int y, string_t text);
    void outTextF(GfxCanvas* can, int f, int x, int y, int lim, string_t text);
    void drawBackground(GfxCanvas* can, GfxRect r);
    void addControl(UIGroup&, GfxRect r, UIWidget& w);
 public:
    WTile(GfxPoint size, int aid = 0, int awhen = 0);
    WTile(int wi, int he, int when = 0);
    void draw(Ptr<GfxCanvas> can, UpdateInfo& ui);
    void setTheme(Theme t);
    void update();
    void update(int);
    const GfxPoint& getSize() const { return size; }
    void getLayoutInfo(LayoutInfo& info);
    bool handleEvent(const UIEvent& e, bool second_pass);
    virtual void addControls(UIGroup&, ObjectHolder&);
};

class WObjectSelectionTile : public WTile {
    GObjectType& type;
    int current;
 public:
    WObjectSelectionTile(GObjectType& atype);
    void         select(int id);
    bool         handleEvent(const UIEvent& e, bool second_pass);

    GObjectType& getType() const { return type; }
    int          getCurrent() const { return current; }
};

class WChartWidget : public UIWidget {
    GChartViewport viewport;
 public:
    WChartWidget();
    ~WChartWidget();
    void draw(Ptr<GfxCanvas> can, UpdateInfo& ui);
    virtual void setCenterPosition(GPoint pt);

    virtual void drawPre(Ptr<GfxCanvas> can);
    virtual void drawPost(Ptr<GfxCanvas> can);

    GPoint getCenterPosition() const { return viewport.center; }
    bool   isValid() const { return true; } // FIXME:  we need some way to disable a chart widget

    const GChartViewport& getViewport() const { return viewport; }
    bool  zoomIn();
    bool  zoomOut();
};

class WScannerChartWidget : public WChartWidget {
    GPoint      scan;
    // FIXME: bubble stuff
    void drawScannerXor(Ptr<GfxCanvas> can);
 protected:
    virtual void drawScanner(Ptr<GfxCanvas> can);
    virtual void sync(bool dblclick);
    static void drawEndBig(GfxCanvas* can, GfxParameters& parm, int x, int y);
    static void drawEndSmall(GfxCanvas* can, GfxParameters& parm, int x, int y);

    void moveKey(int dx, int dy);
    bool handleKey(UIEvent::Key k);
 public:
    WScannerChartWidget();
    ~WScannerChartWidget();
    void drawPre(Ptr<GfxCanvas> can);
    void drawPost(Ptr<GfxCanvas> can);
    void setScannerPosition(GPoint pt);
    void setCenterPosition(GPoint pt);
    bool handleEvent(const UIEvent& e, bool second_pass);

    GPoint getScannerPosition() const { return scan; }

    void getLayoutInfo(LayoutInfo& info);

    enum {
        sc_StyleChange  = st_Last*2,  ///< user can changes style (C-M-letter)
        sc_Zoomable     = st_Last*4,  ///< user can zoom (+/-)
        sc_Keyboard     = st_Last*8,  ///< user can use keyboard
        sc_ShipsOnly    = st_Last*16, ///< only lock on ships
        sc_KeyboardMode = st_Last*32  ///< keyboard mode (Y) active
    };

    Signal0 sig_moved;
};

class WScanner : public UIWidget {
    WScannerChartWidget* chart;
    int sid, xpid, gpid;
 public:
    WScanner();
    ~WScanner();

    void setChart(WScannerChartWidget& achart);
    void update();
    void draw(Ptr<GfxCanvas> can, UpdateInfo& ui);
    void getLayoutInfo(LayoutInfo& info);

    int getScannedPlanetId(bool exact) const { return exact ? xpid : gpid; }
    int getScannedShipId() const { return sid; }
    WScannerChartWidget* getChart() const { return chart; }

};

class WControlScreen : public UIGroup {
    WScanner      scanner;
    int           screen_id;    ///< Screen Id (function key number).
    int           update_when;  ///< Update header when these bits in object change.

    void*         pos_changed_cookie;

    void setGeometry();
    void onPosChanged(int);
 protected:
    GObjectType&  type;
 public:
    ResID         pic_id;
    GPoint        pos;
    string_t      title, desc;
    Signal1<int>  sig_changed;  ///< Raised when displayed object changes.

 private:
    /* tile positioning */
    int   tile_width;       // width of one column
    int   tile_y;           // current y
    int   tile_x;
    int   tile_new_y;       // y for new columns
    int   chart_y2;
    int   chart_x1;

    WScannerChartWidget*      chart;
    UIGroup                   controls;
 protected:
    ObjectHolder holder;
    
    void addChart(WScannerChartWidget& chart);
    void addTile(WTile& tile);
    void addTile(WTile& tile, const char* title);
    void setData(ResID apic, GPoint apos, const string_t& atit, const string_t& adesc);
 public:
    WControlScreen(GObjectType& atype, int screen_id, int update_when);
    ~WControlScreen();

    bool          handleEvent(const UIEvent& e, bool second_pass);
    void          drawInterior(Ptr<GfxCanvas> can, UpdateInfo& ui);
    void          setCurrent(int id);
    GObjectType&  getType() { return type; }
    GPoint        getScannerPosition() const { return chart->getScannerPosition(); }
    WScanner&     getScanner() { return scanner; }
    int           getScreenId() const { return screen_id; }

    virtual void  update() = 0;

    void          updateId(int id);

    int           run();
};

class WControlScreenHeaderTile : public WTile {
    WControlScreen& con;
    int buttons;
 public:
    WControlScreenHeaderTile(WControlScreen& acon, int buttons, int update);
    void drawData(Ptr<GfxCanvas> can);
    void addControls(UIGroup& g, ObjectHolder&);

    enum {
        bName  = 1,              // N
        bSend  = 2,              // I
        bAuto  = 4,              // Auto
        bCycle = 8,              // ,->
        bAdd   = 16              // Add (Ins)
    };
};

class WHeadingTile : public WTile {
    string_t text;
 public:
    WHeadingTile(const string_t& s);
    WHeadingTile(const char* s);
    void drawFixed(Ptr<GfxCanvas> can);
};

int doSelectWindow(WObjectSelectionTile& inter, string_t title);

#endif
