/**
  *  \file client/gdchooser.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_GDCHOOSER_H
#define PCC_V2_CLIENT_GDCHOOSER_H

#include <vector>
#include "ui/listbox.h"
#include "fileio/dirs.h"
#include "cpluslib/sigslot.h"
#include "cpluslib/string.h"
#include "game/player.h"
#include "game/struct.h"
#include "gfx/palette.h"

#define GDC_SCHEMA 2
enum {
#if GDC_SCHEMA == 2
    COLOR_GDC_BACKGROUND    = COLOR_GRAY,
    COLOR_GDC_WINDOW_FRAME  = COLOR_GRAY,
    COLOR_GDC_INACTIVE_TEXT = COLOR_DARK, //BLUEGRAY,// COLOR_GRAYSCALE + 9,
    COLOR_GDC_FRAME         = COLOR_BLACK,
    COLOR_GDC_SELECTED_TEXT = COLOR_BLACK,
    COLOR_GDC_NORMAL_TEXT   = COLOR_BLACK,
    COLOR_GDC_FRAME_TEXT    = COLOR_WHITE
#elif GDC_SCHEMA == 1
    COLOR_GDC_BACKGROUND    = COLOR_SHIELD + 1,
    COLOR_GDC_WINDOW_FRAME  = COLOR_GRAY,
    COLOR_GDC_INACTIVE_TEXT = COLOR_GRAYSCALE + 2,
    COLOR_GDC_FRAME         = COLOR_GRAY,
    COLOR_GDC_SELECTED_TEXT = COLOR_WHITE,
    COLOR_GDC_NORMAL_TEXT   = COLOR_GRAY,
    COLOR_GDC_FRAME_TEXT    = COLOR_GDC_BACKGROUND
#else
    COLOR_GDC_BACKGROUND    = COLOR_GREENSCALE + 1,
    COLOR_GDC_WINDOW_FRAME  = COLOR_GREENSCALE + 4,
    COLOR_GDC_INACTIVE_TEXT = COLOR_GREENSCALE + 3,
    COLOR_GDC_FRAME         = COLOR_GREENSCALE + 15,
    COLOR_GDC_SELECTED_TEXT = COLOR_GREENSCALE + 15,
    COLOR_GDC_NORMAL_TEXT   = COLOR_GREENSCALE + 7,
    COLOR_GDC_FRAME_TEXT    = COLOR_GDC_BACKGROUND
#endif
};

class WGamedirChooser : public UIListbox {
    Ptr<Directory> current;
    enum Action {
        aChdir,
        aParent,
        aShowRoots,
        aRescanWithoutGamelist
    };
    enum ScanArg {
        scanGamelist,
        scanNoGamelist,
        scanRoots
    };
    struct Entry {
        Action          a;
        string_t        name;
        Ptr<Directory>  entry;
        Entry(Action a, const string_t& name, Ptr<Directory> entry);
        ~Entry();
    };
    std::vector<Entry> content;
    void scan(ScanArg arg, Ptr<Directory> dir);
    void scanDirectory(Ptr<Directory> dir);
    static bool isEntryLT(const Entry&, const Entry&);
 public:
    WGamedirChooser(int id = 0);
    WGamedirChooser(Ptr<Directory> current, bool oneup = false, int id = 0);
    ~WGamedirChooser();

    void drawPart(Ptr<GfxCanvas> can, int from, int to);
    void enter();
    void enter(int);

    Ptr<Directory> getDirectory() const { return current; }
    Ptr<Directory> getSubdirectory();
};

class WGameRaceChooser : public UIListbox {
    int            avail;                 // bitfield, 1<<pids
    string_t       infos[NUM_PLAYERS];
    GRaceNameList  list;
 protected:
    virtual void scan(Ptr<Directory> dir) = 0;
    void setInfo(int player, bool avail, string_t text);
 public:
    WGameRaceChooser(int id);
    ~WGameRaceChooser();
    void drawPart(Ptr<GfxCanvas> can, int from, int to);
    bool isAccessible(int index);
    void setDirectory(Ptr<Directory> dir);
};

#endif
