/**
  *  \file client/scr-base.cc
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/screens.h"
#include "client/cscreen.h"
#include "client/marks.h"
#include "game/global.h"
#include "gfx/font.h"
#include "gfx/gfx.h"
#include "gfx/palette.h"
#include "ui/uiglobal.h"
#include "util/nls.h"
#include "ui/msgbox.h"
#include "game/planet.h"
#include "resmgr/cc1resid.h"
#include "cpluslib/format.h"
#include "client/actions.h"
#include "game/ship.h"
#include "ui/listbox.h"
#include "ui/widgets.h"
#include "client/dlg-tech.h"

namespace {
    class WBaseSelect : public WObjectSelectionTile {
     public:
        WBaseSelect();
        void drawData(Ptr<GfxCanvas> can);
    };

    class WBaseScreen : public WControlScreen {
        void* sig_handle;
     public:
        WBaseScreen();
        ~WBaseScreen();
        bool handleEvent(const UIEvent& e, bool second_pass);
        void update();
    };

    class WBaseMineralTile : public WTile {
     public:
        WBaseMineralTile();
        void drawFixed(Ptr<GfxCanvas> can);
        void drawData(Ptr<GfxCanvas> can);
    };

    class WBaseTechTile : public WTile {
     public:
        WBaseTechTile();
        void drawFixed(Ptr<GfxCanvas> can);
        void drawData(Ptr<GfxCanvas> can);
    };

    class WBaseOrderTile : public WTile {
     public:
        WBaseOrderTile();
        void drawFixed(Ptr<GfxCanvas> can);
        void drawData(Ptr<GfxCanvas> can);
        void addControls(UIGroup& g, ObjectHolder& holder);
    };
}

WBaseSelect::WBaseSelect()
    : WObjectSelectionTile(getDisplayedTurn().getBaseType())
{ }

void
WBaseSelect::drawData(Ptr<GfxCanvas> can)
{
    GPlanet& pp = getDisplayedTurn().getPlanet(getCurrent());

    // FIXME: this is very incomplet
    int y = 0, rem = 0;
    parm.just_x = parm.just_y = 0;
    if (getType().isMarked(getCurrent())) {
        drawBackground(can, GfxRect(0, 0, 15, font_heights[FONT_NORMAL]));
        parm.color = standard_colors[COLOR_GREENBLACK];
        drawSelection(can, parm, extent.x + 5, extent.y + 7, 1, 2);
        rem = 15;
    }
    setColor(tc_Static);
    outTextF(can, FONT_NORMAL, rem, y, extent.w - rem, pp.getName());
}

int
chooseBase()
{
    GGameTurn& trn = getDisplayedTurn();
    int nbases = trn.getBaseType().getCount();
    if (nbases == 0) {
        messageBox(_("You do not have any starbases. "
                     "To build starbases, use [F8] on a planet."),
                   _("Starbases"));
        return 0;
    } else if (nbases == 1) {
        return trn.getBaseType().getCurrent();
    } else {
        WBaseSelect selector;
        return doSelectWindow(selector, _("Starbase"));
    }
}

/**************************** WBaseMineralTile ***************************/

WBaseMineralTile::WBaseMineralTile()
    : WTile(GfxPoint(315, font_heights[FONT_NORMAL] * 4))
{ }

void
WBaseMineralTile::drawFixed(Ptr<GfxCanvas> can)
{
    const int y = font_heights[FONT_NORMAL];
    parm.just_x = parm.just_y = 0;
    setColor(tc_Static);

    outText(can, FONT_NORMAL, 5,   0, _("Neu:"));
    outText(can, FONT_NORMAL, 5,   y, _("Tri:"));
    outText(can, FONT_NORMAL, 5, 2*y, _("Dur:"));
    outText(can, FONT_NORMAL, 5, 3*y, _("Mol:"));
    outText(can, FONT_NORMAL, 145, 0, _("Money:"));
    outText(can, FONT_NORMAL, 145, y, _("Supplies:"));
}

void
WBaseMineralTile::drawData(Ptr<GfxCanvas> can)
{
    const int y = font_heights[FONT_NORMAL];
    const int x = 135;
    const int x2 = 135+160;
    GPlanet&  p = getDisplayedTurn().getBaseType().getPlanet();

    parm.just_x = 2;
    parm.just_y = 0;
    setColor(tc_Green);

    outTextF(can, FONT_NORMAL, x,   0, 80, format("%d kt", p.getCargo(el_Neutronium)));
    outTextF(can, FONT_NORMAL, x,   y, 80, format("%d kt", p.getCargo(el_Tritanium)));
    outTextF(can, FONT_NORMAL, x, 2*y, 80, format("%d kt", p.getCargo(el_Duranium)));
    outTextF(can, FONT_NORMAL, x, 3*y, 80, format("%d kt", p.getCargo(el_Molybdenum)));
    outTextF(can, FONT_NORMAL, x2,  0, 90, format("%d mc", p.getCargo(el_Money)));
    outTextF(can, FONT_NORMAL, x2,  y, 78, format("%d kt", p.getCargo(el_Supplies)));
}

/***************************** WBaseTechTile *****************************/

WBaseTechTile::WBaseTechTile()
    : WTile(315, 4 * font_heights[FONT_NORMAL])
{ }

void
WBaseTechTile::drawFixed(Ptr<GfxCanvas> can)
{
    const int y = font_heights[FONT_NORMAL];
    parm.just_x = parm.just_y = 0;
    setColor(tc_Static);

    outText(can, FONT_NORMAL, 5,   0, _("Engines:"));
    outText(can, FONT_NORMAL, 5,   y, _("Hulls:"));
    outText(can, FONT_NORMAL, 5, 2*y, _("Beam Weapons:"));
    outText(can, FONT_NORMAL, 5, 3*y, _("Torpedoes:"));

    outText(can, FONT_NORMAL, 150,   0, _("Defense:"));
    outText(can, FONT_NORMAL, 150,   y, _("Fighters:"));
    outText(can, FONT_NORMAL, 150, 2*y, _("Damage:"));
}

void
WBaseTechTile::drawData(Ptr<GfxCanvas> can)
{
    const int y      = font_heights[FONT_NORMAL];
    const GPlanet& p = getDisplayedTurn().getBaseType().getPlanet();

    parm.just_x = 2;
    parm.just_y = 0;
    setColor(tc_Green);

    outTextF(can, FONT_NORMAL, 140,   0, 22, itoa(p.getBaseTech(tl_Engine)));
    outTextF(can, FONT_NORMAL, 140,   y, 22, itoa(p.getBaseTech(tl_Hull)));
    outTextF(can, FONT_NORMAL, 140, 2*y, 22, itoa(p.getBaseTech(tl_Beam)));
    outTextF(can, FONT_NORMAL, 140, 3*y, 22, itoa(p.getBaseTech(tl_Torpedo)));

    outTextF(can, FONT_NORMAL, 279,   0, 64, itoa(p.getBaseDefense()));
    outTextF(can, FONT_NORMAL, 279,   y, 54, itoa(p.getCargo(el_Fighters)));
    if (p.getBaseDamage()) {
        setColor(tc_Red);
        outTextF(can, FONT_NORMAL, 279, 2*y, 64, itoa(p.getBaseDamage()));
    } else {
        outTextF(can, FONT_NORMAL, 279, 2*y, 64, _("none"));
    }
}

/***************************** WBaseOrderTile ****************************/

WBaseOrderTile::WBaseOrderTile()
    : WTile(315, 100)
{ }

void
WBaseOrderTile::drawFixed(Ptr<GfxCanvas> can)
{
    parm.just_x = parm.just_y = 0;
    setColor(tc_Static);

    outText(can, FONT_NORMAL, 35, 55, _("Mission:"));
    outText(can, FONT_NORMAL, 35, 80, _("Build Fighters/Torpedoes"));
}

void
WBaseOrderTile::drawData(Ptr<GfxCanvas> can)
{
    const GPlanet& p = getDisplayedTurn().getBaseType().getPlanet();
    parm.just_x = parm.just_y = 0;

    /* ship */
    int hull = p.getBaseBuildHull();
    int clnid = p.getShipCloningHere();
    string_t text;

    if (hull) {
        if (clnid) {
            setColor(tc_Red);
            text = format(_("Cloning, and building %s"),
                          getHull(hull).getName());
        } else {
            setColor(tc_Yellow);
            text = format(_("Building a %s"),
                          getHull(hull).getName());
        }
    } else {
        if (clnid) {
            setColor(tc_Yellow);
            text = format(_("Cloning %s"),
                          getDisplayedTurn().getShip(clnid).getName());
        } else {
            setColor(tc_Static);
            text = _("Build a new ship");
        }
    }
    outTextF(can, FONT_NORMAL, 35, 5, extent.w-35, text);

    /* fix/recycle */
    if (p.getBaseShipyardOrder()) {
        setColor(tc_Yellow);
        outTextF(can, FONT_NORMAL, 35, 30, extent.w-35,
                 format(p.getBaseShipyardOrder() == 1
                        ? _("Repairing %s")
                        : _("Recycling %s"),
                        getDisplayedTurn().getShip(p.getBaseShipyardId()).getFullName()));
    } else {
        setColor(tc_Static);
        outTextF(can, FONT_NORMAL, 35, 30, extent.w-35, _("Repair/recycle a starship"));
    }

    /* mission */
    const int mx = 65 + fonts[FONT_NORMAL]->getTextWidth(_("Mission:"));
    setColor(tc_Green);
    outTextF(can, FONT_NORMAL, mx, 55, extent.w-mx, p.getBaseMissionName());
}

void
WBaseOrderTile::addControls(UIGroup& g, ObjectHolder& h)
{
    addControl(g, GfxRect(5,  2, 20, 20), h.add(new UIButton("B", 'b')));
    addControl(g, GfxRect(5, 27, 20, 20), h.add(new UIButton("R", 'r')));
    addControl(g, GfxRect(5, 52, 20, 20), h.add(new UIButton("M", 'm')));
    addControl(g, GfxRect(5, 77, 20, 20), h.add(new UIButton("A", 'a')));
}

/****************************** WBaseScreen ******************************/

WBaseScreen::WBaseScreen()
    : WControlScreen(getDisplayedTurn().getBaseType(), 3, GPlanet::ps_DirtyHead)
{
    addTile(holder.add(new WControlScreenHeaderTile(*this, WControlScreenHeaderTile::bAuto + WControlScreenHeaderTile::bAdd,
                                                    GPlanet::ps_DirtyHead)));
    addTile(holder.add(new WBaseMineralTile()), _("Minerals & Funds:"));
    addTile(holder.add(new WBaseTechTile()), _("Technology & Defense:"));
    addTile(holder.add(new WBaseOrderTile()), _("Orders:"));
    addTile(holder.add(new WPlanetFCodeTile(getDisplayedTurn().getBaseType())));
    addChart(holder.add(new WScannerChartWidget()));

    sig_handle = getDisplayedTurn().sig_planet_changed.add(static_cast<WControlScreen*>(this),
                                                           &WControlScreen::updateId);
}

WBaseScreen::~WBaseScreen()
{
    getDisplayedTurn().sig_planet_changed.remove(sig_handle);
}

static void
doBaseXchg(WControlScreen* scr)
{
    WScanner* scanner = &scr->getScanner();
    int pid = scanner->getScannedPlanetId(scanner->getScannedShipId() != 0);
    if (pid && pid != scr->getType().getCurrent()) {
        if (scr->getType().isExistant(pid))
            putCommand(cm_CS_GotoCS, 3, pid);
        else if (getDisplayedTurn().getPlanetType().isExistant(pid))
            putCommand(cm_CS_GotoCS, 2, pid);
        else
            putKey(ss_Ctrl + SDLK_F1);
    } else {
        putKey(ss_Ctrl + SDLK_F1);
    }
}

bool
WBaseScreen::handleEvent(const UIEvent& e, bool second_pass)
{
    if (e.type == ev_Key) {
        switch (e.key.code) {
         case 'x':
            doBaseXchg(this);
            return true;
         case 'f':
            doPlanetFCodeChange(getDisplayedTurn().getBaseType().getPlanet());
            return true;
         case 'c':
         case 'u':
            doPlanetTransfer(getDisplayedTurn().getBaseType().getPlanet(), e.key.code == 'u');
            return true;
         case SDLK_F8:
            putCommand(cm_CS_GotoCS, 2, getType().getCurrent());
            return true;
         case 'm':
            doBaseChangeMission(getDisplayedTurn().getBaseType().getPlanet());
            return true;
         case 'r':
            doBaseFixRecycle(getDisplayedTurn().getBaseType().getPlanet(), e);
            return true;
         case ss_Ctrl + 'r':
            if (int sid = getDisplayedTurn().getBaseType().getPlanet().getBaseShipyardId())
                putCommand(cm_CS_GotoCS, 1, sid);
            return true;
         case 't':
            doTechUpgradeDialog(getDisplayedTurn().getBaseType().getPlanet());
            return true;

            /* base screen keys:
               'b'    build ship
               'a'    ammo
               't'    tech
               'd'    defense
               's'    ship cost
               ins    sim
               alt-r  reset
               f10    choose fleet */

        }
    }
    return WControlScreen::handleEvent(e, second_pass);
}

void
WBaseScreen::update()
{
    const GPlanet& planet = getDisplayedTurn().getBaseType().getPlanet();
    setData(mkResId(rt_Misc, cc1id_Base_Hi),
            planet.getPos(),
            planet.getName(),
            format(_("(Id #%d)"), planet.getId()));
}

int
doBaseScreen()
{
    return WBaseScreen().run();
}
