/**
  *  \file client/scr-planet.cc
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/actions.h"
#include "client/cscreen.h"
#include "client/dlg-list.h"
#include "client/marks.h"
#include "client/screens.h"
#include "cpluslib/format.h"
#include "game/global.h"
#include "game/objlist.h"
#include "game/planet.h"
#include "game/ship.h"
#include "gfx/font.h"
#include "gfx/gfx.h"
#include "gfx/palette.h"
#include "ui/msgbox.h"
#include "ui/uiglobal.h"
#include "util/misc.h"
#include "util/nls.h"

namespace {
    class WPlanetSelect : public WObjectSelectionTile {
     public:
        WPlanetSelect();
        void drawData(Ptr<GfxCanvas> can);
    };

    class WPlanetScreen : public WControlScreen {
        void* sig_handle;
     public:
        WPlanetScreen();
        ~WPlanetScreen();
        void update();
        bool handleEvent(const UIEvent& e, bool second_pass);
    };

    class WPlanetEconomyTile : public WTile {
     public:
        WPlanetEconomyTile();
        void drawFixed(Ptr<GfxCanvas> can);
        void drawData(Ptr<GfxCanvas> can);
    };

    class WPlanetColonistTile : public WTile {
     public:
        WPlanetColonistTile();
        void drawFixed(Ptr<GfxCanvas> can);
        void drawData(Ptr<GfxCanvas> can);
    };

    class WPlanetNativeTile : public WTile {
     public:
        WPlanetNativeTile();
        void drawFixed(Ptr<GfxCanvas> can);
        void drawData(Ptr<GfxCanvas> can);
    };
}

/***************************** WPlanetSelect *****************************/

WPlanetSelect::WPlanetSelect()
    : WObjectSelectionTile(getDisplayedTurn().getPlanetType())
{ }

void
WPlanetSelect::drawData(Ptr<GfxCanvas> can)
{
    GPlanet& pp = getDisplayedTurn().getPlanet(getCurrent());

    // FIXME: this is very incomplet
    int y = 0, rem = 0;
    parm.just_x = parm.just_y = 0;
    if (getType().isMarked(getCurrent())) {
        drawBackground(can, GfxRect(0, 0, 15, font_heights[FONT_NORMAL]));
        parm.color = standard_colors[COLOR_GREENBLACK];
        drawSelection(can, parm, extent.x + 5, extent.y + 7, 1, 2);
        rem = 15;
    }
    setColor(tc_Static);
    outTextF(can, FONT_NORMAL, rem, y, extent.w - rem, pp.getName());
}

int
choosePlanet()
{
    GGameTurn& trn = getDisplayedTurn();
    int nplanets = trn.getPlanetType().getCount();
    if (nplanets == 0) {
        messageBox(_("You do not have any planets. "
                     "Unload colonists from a starship to a planet to colonize it."),
                   _("Planets"));
        return 0;
    } else if (nplanets == 1) {
        return trn.getPlanetType().getCurrent();
    } else {
        WPlanetSelect selector;
        return doSelectWindow(selector, _("Planet"));
    }
}

/*************************** WPlanetEconomyTile **************************/

WPlanetEconomyTile::WPlanetEconomyTile()
    : WTile(GfxPoint(315, font_heights[FONT_NORMAL] * 8))
{ }

void
WPlanetEconomyTile::drawFixed(Ptr<GfxCanvas> can)
{
    const int y = font_heights[FONT_NORMAL];

    setColor(tc_Static);
    parm.just_x = parm.just_y = 0;
    outText(can, FONT_NORMAL, 5,   0, _("Mineral Mines:"));
    outText(can, FONT_NORMAL, 5,   y, _("Factories:"));
    outText(can, FONT_NORMAL, 5, 2*y, _("Defense Posts:"));
    outText(can, FONT_NORMAL, 5, 3*y, _("Neutronium:"));
    outText(can, FONT_NORMAL, 5, 4*y, _("Tritanium:"));
    outText(can, FONT_NORMAL, 5, 5*y, _("Duranium:"));
    outText(can, FONT_NORMAL, 5, 6*y, _("Molybdenum:"));
    outText(can, FONT_NORMAL, 5, 7*y, _("Supplies:"));
}

void
WPlanetEconomyTile::drawData(Ptr<GfxCanvas> can)
{
    const int y = font_heights[FONT_NORMAL];
    const int x1 = 5+160, x2 = x1 + 24;   // FIXME
    const int wi1 = 50, wi2 = 90;       // FIXME
    const GPlanet& p = getDisplayedTurn().getPlanetType().getPlanet();

    setColor(tc_Green);
    parm.just_x = 2;
    parm.just_y = 0;
    outTextF(can, FONT_NORMAL, x1,   0, wi1, itoa(p.getMines()));
    outTextF(can, FONT_NORMAL, x1,   y, wi1, itoa(p.getFactories()));
    outTextF(can, FONT_NORMAL, x1, 2*y, wi1, itoa(p.getDefense()));
    outTextF(can, FONT_NORMAL, x2, 3*y, wi2, format(_("%d kt"), p.getCargo(el_Neutronium)));
    outTextF(can, FONT_NORMAL, x2, 4*y, wi2, format(_("%d kt"), p.getCargo(el_Tritanium)));
    outTextF(can, FONT_NORMAL, x2, 5*y, wi2, format(_("%d kt"), p.getCargo(el_Duranium)));
    outTextF(can, FONT_NORMAL, x2, 6*y, wi2, format(_("%d kt"), p.getCargo(el_Molybdenum)));
    outTextF(can, FONT_NORMAL, x2, 7*y, wi2, format(_("%d kt"), p.getSupplies()));
}

/************************** WPlanetColonistTile **************************/

WPlanetColonistTile::WPlanetColonistTile()
    : WTile(315, 4 * font_heights[FONT_NORMAL])
{ }

void
WPlanetColonistTile::drawFixed(Ptr<GfxCanvas> can)
{
    const int y = font_heights[FONT_NORMAL];
    const int wi = 95;

    setColor(tc_Static);
    parm.just_x = 0;
    parm.just_y = 0;
    outTextF(can, FONT_NORMAL, 5,   0, wi, _("Population:"));
    outTextF(can, FONT_NORMAL, 5,   y, wi, _("Tax Rate:"));
    outTextF(can, FONT_NORMAL, 5, 2*y, wi, _("Colonists are"));
}

void
WPlanetColonistTile::drawData(Ptr<GfxCanvas> can)
{
    const int y = font_heights[FONT_NORMAL];
    const int x = 100;
    const int wi = extent.w - x;
    const GPlanet& p = getDisplayedTurn().getPlanetType().getPlanet();

    setColor(tc_Green);
    parm.just_x = 0;
    parm.just_y = 0;
    outTextF(can, FONT_NORMAL, x,   0, wi, clansToString(p.getColonists()));
    outTextF(can, FONT_NORMAL, x,   y, wi, format("%d%%", p.getColonistTax()));

    int happy = p.getColonistHappiness();
    if (happy <= 40)
        setColor(tc_Red);
    else if (happy < 70)
        setColor(tc_Yellow);
    outTextF(can, FONT_NORMAL, x, 2*y, wi, format("%s (%d)", p.getHappyStr(happy), happy));
}

/*************************** WPlanetNativeTile ***************************/

WPlanetNativeTile::WPlanetNativeTile()
    : WTile(315, 4 * font_heights[FONT_NORMAL])
{ }

void
WPlanetNativeTile::drawFixed(Ptr<GfxCanvas> /*can*/)
{ }

void
WPlanetNativeTile::drawData(Ptr<GfxCanvas> can)
{
    const GPlanet& p = getDisplayedTurn().getPlanetType().getPlanet();
    const int y = font_heights[FONT_NORMAL];

    setColor(tc_Static);
    parm.just_x = parm.just_y = 0;
    
    if (!p.getNativeRace()) {
        outTextF(can, FONT_NORMAL, 5, 0, extent.w-5, _("There are no natives on this planet."));
        drawBackground(can, GfxRect(5, y, extent.w-5, 3*y));
    } else {
        outTextF(can, FONT_NORMAL, 5,   0, 95, _("Race:"));
        outTextF(can, FONT_NORMAL, 5,   y, 95, _("Population:"));
        outTextF(can, FONT_NORMAL, 5, 2*y, 95, _("Tax Rate:"));
        outTextF(can, FONT_NORMAL, 5, 3*y, 95, _("Natives are"));

        const int x = 100, wi = extent.w - 100;
        setColor(tc_Green);
        outTextF(can, FONT_NORMAL, x,   0, wi,
                 string_t(p.getNativeRaceName(p.getNativeRace())) + ", " + p.getNativeGovernmentName(p.getNativeGovernment()));
        outTextF(can, FONT_NORMAL, x,   y, wi, clansToString(p.getNatives()));
        outTextF(can, FONT_NORMAL, x, 2*y, wi, format("%d%%", p.getNativeTax()));
        int happy = p.getNativeHappiness();
        if (happy <= 40)
            setColor(tc_Red);
        else if (happy < 70)
            setColor(tc_Yellow);
        outTextF(can, FONT_NORMAL, x, 3*y, wi, format("%s (%d)", p.getHappyStr(happy), happy));
    }
}

/**************************** WPlanetFCodeTile ***************************/

WPlanetFCodeTile::WPlanetFCodeTile(GObjectType& type)
    : WTile(315, 4*font_heights[FONT_NORMAL]), type(type)
{ }

void
WPlanetFCodeTile::drawFixed(Ptr<GfxCanvas> can)
{
    const int y = font_heights[FONT_NORMAL];
    
    parm.just_x = parm.just_y = 0;
    setColor(tc_Static);
    outText(can, FONT_NORMAL, 5, 0, _("Funds:"));
    outText(can, FONT_NORMAL, 5, y, _("Friendly Code:"));
}

void
WPlanetFCodeTile::drawData(Ptr<GfxCanvas> can)
{
    const int y = font_heights[FONT_NORMAL];
    GPlanet& p = getDisplayedTurn().getPlanet(type.getCurrent());
    
    parm.just_x = parm.just_y = 0;
    setColor(tc_Green);
    outTextF(can, FONT_NORMAL, 65, 0, 170, format("%d mc", p.getCargo(el_Money)));
    outTextF(can, FONT_NORMAL, 125, y, 70, p.getFCode());

    string_t comment = "FIXME"; // FIXME
    if (comment.length()) {
        setColor(tc_Yellow);
        outTextF(can, FONT_NORMAL, 5, 5*y/2, extent.w-5, comment);
    } else {
        setColor(tc_Static);
        outTextF(can, FONT_NORMAL, 5, 5*y/2, extent.w-5, _("(press [F9] to add a note)"));
    }
}

/***************************** WPlanetScreen *****************************/

WPlanetScreen::WPlanetScreen()
    : WControlScreen(getDisplayedTurn().getPlanetType(), 2, GPlanet::ps_DirtyHead)
{
    addTile(holder.add(new WControlScreenHeaderTile(*this, WControlScreenHeaderTile::bAuto + WControlScreenHeaderTile::bAdd + WControlScreenHeaderTile::bSend,
                                                    GPlanet::ps_DirtyHead)));
    addTile(holder.add(new WPlanetEconomyTile()), _("Economy:"));
    addTile(holder.add(new WPlanetNativeTile()), _("Natives:"));
    addTile(holder.add(new WPlanetColonistTile()), _("Colonists:"));
    addTile(holder.add(new WPlanetFCodeTile(getDisplayedTurn().getPlanetType())));
    addChart(holder.add(new WScannerChartWidget()));

    sig_handle = getDisplayedTurn().sig_planet_changed.add(static_cast<WControlScreen*>(this),
                                                           &WControlScreen::updateId);
}

WPlanetScreen::~WPlanetScreen()
{
    getDisplayedTurn().sig_planet_changed.remove(sig_handle);
}

void
WPlanetScreen::update()
{
    int pid = getDisplayedTurn().getPlanetType().getCurrent();
    const GPlanet& planet = getDisplayedTurn().getPlanet(pid);
    setData(mkResId(rt_Climate, planet.getTemp()),
            planet.getPos(),
            planet.getName(),
            format(_("(Id #%d, %s - %dF)"),
                   pid, GPlanet::getTempStr(planet.getTemp()), planet.getTemp()));
}

static void
doPlanetXchg(WPlanetScreen* scr)
{
    WScanner* scanner = &scr->getScanner();
    int pid = scanner->getScannedPlanetId(scanner->getScannedShipId() != 0);
    if (pid && pid != scr->getType().getCurrent() && scr->getType().isExistant(pid))
        putCommand(cm_CS_GotoCS, 2, pid);
    else
        putKey(ss_Ctrl + SDLK_F1);
}

bool
WPlanetScreen::handleEvent(const UIEvent& e, bool second_pass)
{
    if (e.type == ev_Key) {
        switch(e.key.code) {
         case 'x':
            doPlanetXchg(this);
            return true;
         case 'c':
         case 'u':
            doPlanetTransfer(getDisplayedTurn().getPlanetType().getPlanet(), e.key.code == 'u');
            return true;
         case 'f':
            doPlanetFCodeChange(getDisplayedTurn().getPlanetType().getPlanet());
            return true;

            /* planet screen keys:
               f8       build/go to base
               f9       comment
               b, d, m  build/defense/mine
               t        tax
               ins      sim
               i        mail
               alt-r    reset
               c-s      undo supply sale
               s        sell supplies
               g        give */
        }
    }
    return WControlScreen::handleEvent(e, second_pass);
}

int
doPlanetScreen()
{
    return WPlanetScreen().run();
}
