/**
  *  \file vcranim.h
  *  \brief VCR Animation
  *
  *  VCR Sprites
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_VCRANIM_H
#define PCC_V2_VCRANIM_H

#include "gfx/anim.h"
#include "game/vcr.h"
#include "sound/sound.h"

class VcrExplSprite : public GfxAniSprite {
    int dx, dy;
    int sumx, sumy;
 public:
    enum {
        EXPLODE_FRAMES = 64+10,
        EXPLODE_SIZE   = 80,
        MAX_HOTSPOTS   = 64,
        EXPLODE_SIZE2  = EXPLODE_SIZE*EXPLODE_SIZE
    };
    static Ptr<GfxAnimation> getExplosion(int size, int speed);
    VcrExplSprite(int x, int y, int layer, int size, int speed);
    VcrExplSprite(int x, int y, int layer, int size, int speed, int adx, int ady);

    void tick();
};

class VcrShieldSprite : public GfxAniSprite {
 public:
    static Ptr<GfxAnimation> getShield(int dir);
    VcrShieldSprite(int x, int y, int layer, int dir);
};

class VcrFighterSprite : public GfxSprite {
    enum {
        WIDTH = 9,
        HEIGHT = 7
    };
    int side, track, player;
    Ptr<VcrPlayer> vcr_player;
    Ptr<GfxPixmap> attack, retract;

    int target_x, target_y;
    int status;
    
    VcrPlayer::time_type last_time;
    
    static int getTrackY(int tr);
 public:
    struct Status {
        int status, x, y, target_x, target_y;
    };

    VcrFighterSprite(const Status& st, int alayer,
                     int aside, int atrack, int aplayer,
                     Ptr<VcrPlayer> aplay,
                     Ptr<GfxPixmap> aatt, Ptr<GfxPixmap> aretr);
    VcrFighterSprite(int x, int y,
                     int alayer,
                     int aside, int atrack, int aplayer,
                     Ptr<VcrPlayer> avcr_player,
                     Ptr<GfxPixmap> aatt, Ptr<GfxPixmap> aretr);
    ~VcrFighterSprite();
    void tick();
    void draw(Ptr<GfxCanvas> can);
    void kill();

    void storeStatus(Status& st);
};

class VcrWeaponSprite : public GfxSprite {
 protected:
    int x1, y1;
    VcrWeaponSprite(GfxRect apos, int alayer, int x1, int y1);
    Ptr<GfxSprite> makeBang(VcrPlayer* ply, int direction, int side);
 public:
    ~VcrWeaponSprite();
    virtual void setBang(VcrPlayer* ply, int side) = 0;
    
    static int active_fx_sprites;
};

class VcrBeamSprite : public VcrWeaponSprite {
    int scheme, status;
    int x0, y0;

    static int getScheme(int damage, int kill);
 public:
    VcrBeamSprite(int ax0, int ay0, int ax1, int ay1,
                  int alayer, int damage, int kill);
    ~VcrBeamSprite();
    void tick();
    void draw(Ptr<GfxCanvas> can);
    void setBang(VcrPlayer* ply, int side);
};

class VcrTorpSprite : public VcrWeaponSprite {
    int x0, y0;
    Ptr<GfxPixmap> pix;
    int hit_side;
    int status;
    Ptr<GfxSprite> bang;
    SfxSample bang_sound;
 public:
    VcrTorpSprite(int ax0, int ay0, int ax1, int ay1,
                  int alayer, Ptr<GfxPixmap> apix, int ahits,
                  SfxSample bang_sound);
    ~VcrTorpSprite();
    void tick();
    void draw(Ptr<GfxCanvas> can);
    void setBang(VcrPlayer* ply, int side);
};

class VcrTimeSprite : public GfxSprite {
    VcrPlayer::time_type current;
    Ptr<VcrPlayer> vcr_player;
 public:
    VcrTimeSprite(int xr, int yb, int alayer, Ptr<VcrPlayer> aply);
    void tick();
    void draw(Ptr<GfxCanvas> can);
};

class VcrDistanceSprite : public GfxSprite {
    long current;
    Ptr<VcrPlayer> vcr_player;
 public:
    VcrDistanceSprite(int xl, int yb, int alayer, Ptr<VcrPlayer> aply);
    void tick();
    void draw(Ptr<GfxCanvas> can);
};

class VcrResultSprite : public GfxSprite {
    Ptr<VcrPlayer> vcr_player;
    TVcr& vcr;
    int res;
 public:
    VcrResultSprite(GfxRect pos, int alayer, Ptr<VcrPlayer> ply, TVcr& vcr);
    void tick();
    void draw(Ptr<GfxCanvas> can);
};

/* Sound */

enum {
    VCR_SfxImportance = 10
};

inline uint8 panFromX(int x) { return x * 255 / 640; }

#endif
