 /**
  *  \file vcrplay.h
  *  \brief VCR Player (GUI)
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_VCRPLAY_H
#define PCC_V2_VCRPLAY_H

#include "cpluslib/smartptr.h"
#include "cpluslib/call.h"
#include "ui/ui.h"
#include "game/vcr.h"
#include "game/vcrdb.h"
#include "ui/commands.h"

enum {
    cm_VCR_Prev = cm_VcrSelectionFirst,
    cm_VCR_Next,
    cm_VCR_LeftInfo,
    cm_VCR_RightInfo,
    cm_VCR_SetBattle
};

/** VCR Selector Widget.
    \cmd cm_VCR_Prev: switch to previous VCR
    \cmd cm_VCR_Next: switch to next VCR */
class WVcrSelector : public UIWidget {
    int index;                  /**< Currently displayed battle.
                                     Range is [0, db->getVcrCount()) */
    Ptr<VcrDatabase> db;        /**< VCR Database to play. */
    Ptr<VcrEntry> entry;        /**< Current database entry. Equals
                                     db->getEntry(index). */
    Ptr<VcrPlayer> player;      /**< VCR Player to use. */
    TVcr vcr;                   /**< Currently displayed VCR. Equals
                                     entry->getVcr(). */
    int disp_status;            /**< Displayed status. 0 if battle ending
                                     not known, nonzero else. */
    bool playing;

    GfxParameters parm;

    void drawWarrior(Ptr<GfxCanvas> can, int x, int y, int w, int side);
    void drawStatus(Ptr<GfxCanvas> can);
    void maybeRestartPlayer();
 public:
    WVcrSelector(Ptr<VcrDatabase> adb);
    ~WVcrSelector();

    void selectVcr(int aindex);
    void draw(Ptr<GfxCanvas> can, UpdateInfo& ui);
    void getLayoutInfo(LayoutInfo&);

    bool handleEvent(const UIEvent& e, bool second_pass);

    /* slots */
    void playIt(int dummy=0);
    void next(int dummy=0);
    void prev(int dummy=0);
};

#endif
