/**
  *  \file fileio/iostr.h
  *
  *  #Stream / iostream bridge.
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_FILEIO_IOSTR_H
#define PCC_V2_FILEIO_IOSTR_H

#include <iostream>
#include "fileio/streams.h"
#include "cpluslib/smartptr.h"
#include "defs.h"

class StandardStreambuf : public std::streambuf {
#ifndef HAVE_STREAM_TYPES
    /* GNU libio lacks these */
    typedef long pos_type, off_type;
    typedef ios ios_base;
#else
    typedef std::ios_base ios_base;
#endif
    Ptr<Stream> stream;
    char* buffer;
    int buf_len;
    void create_buffer();
 public:
    StandardStreambuf();
    StandardStreambuf(Ptr<Stream> astream);
    ~StandardStreambuf();
    bool is_open();
    StandardStreambuf* open(Ptr<Stream> astream);
    StandardStreambuf* close();
    
    int overflow(int);
    int underflow();
    int sync();
    pos_type seekoff(off_type off, ios_base::seekdir way,
                     ios_base::openmode which);
    pos_type seekpos(pos_type sp, ios_base::openmode which);
};

class StandardStream : public std::iostream {
    StandardStreambuf buf;
 public:
    StandardStream();
    StandardStream(Ptr<Stream> astream);
    ~StandardStream();
    void open(Ptr<Stream> astream);
    void close();
    StandardStreambuf* rdbuf() { return &buf; }
};

#endif
