/**
  *  \file fileio/rle.h
  *  \brief Run-Length Encoded Data
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_FILEIO_RLE_H
#define PCC_V2_FILEIO_RLE_H

#include "fileio/streams.h"
#include "cpluslib/smartptr.h"
#include "cpluslib/types.h"

class RleReaderStream : public Stream {
    Buffer<char> buffer;        ///< Compressed data.
    uint32 size;                ///< Size of uncompressed data.
    uint32 pos;                 ///< Current position.
    uint8 counter;              ///< Decompression: repeat counter.
    char chr;                   ///< Decompression: repeated character.
    char code;                  ///< Decompression: compression code.
    std::size_t index;          ///< Index into buffer.
    int32 chunk_len;            ///< Remaining bytes (uncompressed) in this chunk.
    bool eof_reached;           ///< True iff EOF reached.
    string_t name;              ///< Name (just for reference).

    inline char get();
 public:
    RleReaderStream(Ptr<Stream> abase);
    ~RleReaderStream();

    std::size_t read(char* buffer, std::size_t max);
    std::size_t write(const char* buffer, std::size_t max);
    int getCapabilities() const;
    bool seek(fpos_t pos);
    fpos_t getPos();
    fpos_t getSize();
    string_t getName();
};

#endif
