/**
  *  \file game/checksum.cc
  *  \brief Checksum Routines
  *
  *  Standard VGA Planets checksum: sum of unsigned bytes. These are
  *  used by Tim's host and clients.
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "game/checksum.h"

/** Compute checksum over a data block.
    Most checksums are just that: the sum of all bytes of the block. */
int32
getChecksum(const char* data, std::size_t length)
{
    int32 sum = 0;
    while (length--)
        sum += static_cast<unsigned char>(*data++);
    return sum;
}

int32
getChecksum(uint16 value)
{
    return (value & 255) + (value >> 8);
}
