/**
  *  \file game/checksum.h
  *  \brief Checksum Routines
  *
  *  Various checksum routines needed to validate game data.
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_CHECKSUM_H
#define PCC_V2_GAME_CHECKSUM_H

#include <cstddef>
#include "cpluslib/types.h"

/* checksum.cc */
int32 getChecksum(const char* data, std::size_t length);
int32 getChecksum(uint16 value);

/* checksum-ctl.cc */
int32 getControlEntry(int otyp, int oid);
void  setControlEntry(int otyp, int oid, int32 checksum);

int   loadControl(int player);
void  saveControl(int type);

void  initControl();
void  doneControl();

/* checksum-crc.cc */
uint32 getDigest(const char* data, std::size_t length, uint32 value = 0);

#endif
