/**
  *  \file game/cmds.h
  *  \brief Auxiliary Command Management
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_CMDS_H
#define PCC_V2_GAME_CMDS_H

#include <list>

#include "cpluslib/string.h"

class GCommand {
 public:
    enum Type {
        phc_Language,               /* 0            name */
        phc_SendConfig,             /* 0            "" */
        phc_SendRacenames,          /* 0            "" */
        phc_SetRaceName,            /* phcrn_XXXX   name */
        phc_Filter,                 /* 0            "yes"|"no" */
        phc_ConfigAlly,             /* race         code string */
        phc_AddDropAlly,            /* race         "add"|"drop" */
        phc_GiveShip,               /* sid          race */
        phc_GivePlanet,             /* pid          race */
        phc_RemoteControl,          /* sid          "control"|"allow"|"forbid"|"drop" */
        phc_RemoteDefault,          /* 0            "allow"|"forbid" */
        phc_Beamup,                 /* sid          code string */

        phc_TAlliance,              /* 0            fcode list (3*N chars) */

        phc_SendFcodes,             /* 0            "" */
        phc_Other,                  /* 0            command */
        phc_LAST = phc_Other
    };

    /* "id" parameters for phc_SetRaceName */
    enum {
        phcrn_Long,
        phcrn_Short,
        phcrn_Adjective
    };

 private:
    Type      cmd;
    int       id;
    string_t  arg;
 public:
    GCommand(Type acmd, int aid, string_t aarg)
        : cmd(acmd), id(aid), arg(aarg) { }
    ~GCommand();

    Type             getCommand() const { return cmd; }
    int              getId()      const { return id; }
    const string_t&  getArg()     const { return arg; }

    // FIXME: do we need more setters?
    void setArg(const string_t& s) { arg = s; }

    string_t getCommandText() const;

    static GCommand* parseCommand(string_t text, bool fromfile);
    static bool      isMessageIntroducer(string_t text);
};

class GCommandContainer {
    typedef std::list<GCommand*> command_list;
    command_list cmds;
    command_list::iterator findCommand(GCommand::Type typ, int id);
 public:
    GCommandContainer();
    ~GCommandContainer();

    void clear();

    const GCommand* getCommand(GCommand::Type typ, int id);
    const GCommand* setCommand(GCommand::Type typ, int id, string_t arg);
    const GCommand* setCommand(GCommand* cmd);
    bool            removeCommand(GCommand::Type typ, int id);

    typedef command_list::const_iterator iterator;
    iterator begin() const { return cmds.begin(); }
    iterator end() const { return cmds.end(); }
};

#endif
