/**
  *  \file game/fcode-list.cc
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <algorithm>
#include <cctype>
#include "game/fcode.h"
#include "cpluslib/string.h"

static bool
compareFCodes(const GFCode& l, const GFCode& r)
{
    using namespace std;
    string_t ls = l.getFCode(), rs = r.getFCode();

    if (ls.length() && rs.length())
        if (isalnum(ls[0]) && !isalnum(rs[0]))
            return true;
        else if (!isalnum(ls[0]) && isalnum(rs[0]))
            return false;
    
    int i = strCaseCmp(ls, rs);
    if (i == 0)
        return ls < rs;
    else
        return i < 0;
}

/******************************* GFCodeList ******************************/

/** Static instance (the fcodes.cc file). */
GFCodeList GFCodeList::instance;

/** Make empty list. */
GFCodeList::GFCodeList()
{ }

/** Make sublist of some other list. The new list will contain all
    friendly codes valid for object o. */
GFCodeList::GFCodeList(const GFCodeList& l, const GObject& o)
{
    for (iterator i = l.begin(); i != l.end(); ++i)
        if (i->worksOn(o))
            addFCode(*i);
    sort();
}

GFCodeList::~GFCodeList()
{ }

/** Add a friendly code. You should use sort as soon as possible to
    re-sort the list. */
void
GFCodeList::addFCode(const GFCode& code)
{
    data.push_back(code);
}

void
GFCodeList::sort()
{
    /* FIXME: use stable_sort instead? */
    std::sort(data.begin(), data.end(), compareFCodes);
}

void
GFCodeList::clear()
{
    data.clear();
}

/** Look up friendly code by name.
    \returns iterator pointing to friendly code, or end() if none */
GFCodeList::iterator
GFCodeList::getFCodeByName(string_t fc) const
{
    iterator i = begin();
    while (i != end() && i->getFCode() != fc)
        ++i;
    return i;
}

/** True iff friendly code is special. A friendly code is special if
    it is on the global FC list and not marked unspecial. */
bool
GFCode::isSpecial(const string_t& fc)
{
    GFCodeList::iterator i = GFCodeList::getInstance().getFCodeByName(fc);
    return i != GFCodeList::getInstance().end()
        && (i->getFlags() & fc_Unspecial) == 0;
}
