/**
  *  \file game/fcode.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_FCODE_H
#define PCC_V2_GAME_FCODE_H

#include <vector>
#include <iosfwd>
#include "cpluslib/string.h"
#include "game/player.h"
#include "game/obj.h"

class GPlanet;
class GShip;
class Stream;

class GFCode {
    char          code[3];
    string_t      description;
    int           players;
    int           flags;
 public:
    GFCode();
    GFCode(string_t c, string_t desc_line);
    ~GFCode();

    string_t getFCode() const { return string_t(code, 3); }
    string_t getDescription() const { return expandRaceNames(description); }
    int      getFlags() const { return flags; }
    int      getPlayerMask() const { return players; }

    bool worksOn(const GObject& o) const;
    bool worksOn(const GShip& s) const;
    bool worksOn(const GPlanet& p) const;

    enum {
        fc_Ship       = 1,      ///< Works on ships.
        fc_Planet     = 2,      ///< Works on planets.
        fc_Base       = 4,      ///< Works on starbases
        fc_Capital    = 8,      ///< Works on capital ships only.
        fc_Alchemy    = 16,     ///< Works on alchemy ships only.
        fc_Registered = 32,     ///< Works for registered players only.
        fc_Unspecial  = 64      ///< Not a special friendly code per se.
    };

    static bool isNumeric(const string_t& fc);
    static bool isExtraFC(const string_t& fc);
    static bool isSpecial(const string_t& fc);
    static int  getNumericValue(const string_t& fc);

    static bool isAllowedRandom(const string_t& fc);
    static string_t getRandomFCode();

    static void clearExtraFC();
    static void loadExtraFC(Stream*);
};

class GFCodeList {
    typedef std::vector<GFCode> container_t;
    container_t data;
    
    static GFCodeList instance;
 public:
    typedef container_t::const_iterator  iterator;

    GFCodeList();
    GFCodeList(const GFCodeList&, const GObject& o);
    ~GFCodeList();

    /* container interface */
    container_t::size_type getSize() const { return data.size(); }
    iterator               begin() const { return data.begin(); }
    iterator               end() const { return data.end(); }

    /* manipulator interface */
    void addFCode(const GFCode& code);
    void sort();
    void clear();

    void loadFromFile(std::istream& in, const string_t& name);

    /* accessor interface */
    const GFCode& operator[](container_t::size_type n) const { return data[n]; }
    iterator      getFCodeByName(string_t fc) const;

    static GFCodeList& getInstance() { return instance; }
};

void initFCodes();

#endif
