/**
  *  \file game/ion.cc
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "game/ion.h"
#include "util/nls.h"
#include "game/global.h"
#include "cpluslib/format.h"

GIonStorm::GIonStorm(int id)
    : id(id)
{
    data.volt = 0;              // non-existant
}

GIonStorm::~GIonStorm()
{ }

void
GIonStorm::addKoreData(const TKoreStorm& kstorm)
{
    data = kstorm;
    grow = (kstorm.volt & 1) != 0;
}

string_t
GIonStorm::getName() const
{
    return name;
}

string_t
GIonStorm::getFullName() const
{
    return format(_("%s (Ion storm #%d)"), name, id);
}

int
GIonStorm::getId() const
{
    return id;
}

int
GIonStorm::getOwner() const
{
    return 0;
}

bool
GIonStorm::isExistant() const
{
    return data.volt != 0;
}

GPoint
GIonStorm::getPos() const
{
    return GPoint(data.x, data.y);
}

int
GIonStorm::getRadius() const
{
    return data.radius;
}

/***************************** GIonStormType *****************************/

GIonStormType::GIonStormType(GGameTurn& atrn)
    : GObjectType(NUM_STORMS, atrn)
{ }

bool
GIonStormType::isExistant(int index)
{
    return index > 0 && index <= NUM_STORMS && trn.getIonStorm(index).isExistant();
}

bool
GIonStormType::isMarked(int /*index*/)
{
    return false;
}

void
GIonStormType::toggleMark(int /*index*/)
{ }

int
GIonStormType::getDirtyFlags(int /*index*/)
{
    return 0;
}

string_t
GIonStormType::getName(int index)
{
    return trn.getIonStorm(index).getFullName();
}

string_t
GIonStormType::getClassName()
{
    return string_t(_("Ion Storm"));
}

GIonStorm&
GIonStormType::getObject(int index)
{
    return trn.getIonStorm(index);
}
