/**
  *  \file game/ion.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_ION_H
#define PCC_V2_GAME_ION_H

#include "game/obj.h"
#include "game/struct.h"
#include "game/objtypes.h"

class GIonStorm : public GCircularObject {
    TKoreStorm data; /* contains almost everything we need except for... */
    bool       grow;
    string_t   name;
    int        id;
 public:
    GIonStorm(int id);
    ~GIonStorm();

    /* Loading */
    void     setName(string_t name) { this->name = name; }
    void     addKoreData(const TKoreStorm& data);

    /* GCircularObject interface */
    string_t getName() const;
    string_t getFullName() const;
    int      getId() const;
    int      getOwner() const;
    bool     isExistant() const;
    GPoint   getPos() const;
    int      getRadius() const;

    /* Ion storm accessors */
    int      getVoltage() const { return data.volt; }
    bool     isGrowing() const { return grow; }
    int      getHeading() const { return data.heading; }
    int      getSpeed() const { return data.speed; }
};

class GIonStormType : public GObjectType {
 public:
    GIonStormType(GGameTurn& atrn);
    
    bool isExistant(int index);
    bool isMarked(int index);
    void toggleMark(int index);

    int  getDirtyFlags(int index);
    string_t getName(int index);
    string_t getClassName();

    GIonStorm& getObject(int index);
};

#endif
