/**
  *  \file game/load.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_LOAD_H
#define PCC_V2_GAME_LOAD_H

#include "cpluslib/smartptr.h"
#include "cpluslib/types.h"
#include "fileio/dirs.h"
#include "fileio/streams.h"
#include "game/struct.h"
#include "cpluslib/assert.h"

class GGameTurn;

enum {
    WHERE_DAT = 1,
    WHERE_DIS = 2
};

class GDirectoryOverview {
    int pp_flags[NUM_PLAYERS];
    int global_flags;
 public:
    GDirectoryOverview();

    /* per-player flags */
    enum {
        have_RST      = 1,       ///< We have a current RST
        have_TRN      = 2,       ///< We have a current TRN
        have_Gamedir  = 4,       ///< We have a current game directory
        have_NewRST   = 8,       ///< We have a new RST (along with game dir)
        have_Conflict = 16
    };
    void scan(Directory* dir);
    int getPlayerFlags(int pid) const { ASSERT(pid > 0 && pid <= NUM_PLAYERS); return pp_flags[pid-1]; }
    int getGlobalFlags() const { return global_flags; }
};

/* single-load routines, load.cc */
void loadShips(GGameTurn& trn, Ptr<Stream> s, int count, int where, int asrc);
void loadPlanets(GGameTurn& trn, Ptr<Stream> s, int count, int where, int asrc);
void loadBases(GGameTurn& trn, Ptr<Stream> s, int count, int where, int asrc);
void loadTargets(GGameTurn& trn, Ptr<Stream> s, int count, bool decode);
void loadShipXY(GGameTurn& trn, Ptr<Stream> s, int avail, int reject);
void loadKoreMinefields(GGameTurn& trn, Ptr<Stream> s, int count);
void loadKoreIonStorms(GGameTurn& trn, Ptr<Stream> s, int count);
void addMessageFromFile(GGameTurn& trn, string_t text, int sender, uint16 recv);

/* complete routines, load-{rst,trn,dir}.cc */
void loadResult(GGameTurn& trn, Ptr<Stream> s, int player);
void loadTurn(GGameTurn& trn, Ptr<Stream> s, int player);
void loadDirectory(GGameTurn& trn, Ptr<Directory> d, int player);

#endif
