/**
  *  \file game/lock.h
  *  \brief Locking onto Objects
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_LOCK_H
#define PCC_V2_GAME_LOCK_H

#include "game/coord.h"

class GGameTurn;

enum TLockMode {
    lm_Left,
    lm_Right,
    lm_MAX
};

enum TLockItems {
    li_Planet    = 1,
    li_Ship      = 2,
    li_Ufo       = 4,
    li_Marker    = 8,
    li_Minefield = 16
};

class GLockData {
    GPoint clicked;
    GPoint min, max;
    GPoint found;
    bool   marked;
    long   min_dist;
 public:
    GLockData(GPoint clicked, bool marked);
    GLockData(GPoint clicked, GPoint min, GPoint max, bool marked);
    void   addPoint(GPoint pt, bool marked);
    GPoint getPoint() const { return found; }
};

void findPlanet(GGameTurn& trn, GLockData& data);
void findShip(GGameTurn& trn, GLockData& data);
void findUfo(GGameTurn& trn, GLockData& data);
void findMarker(GGameTurn& trn, GLockData& data);
void findMinefield(GGameTurn& trn, GLockData& data);

void setLockMode(TLockMode mode, int items);
int  getLockMode(TLockMode mode);
void findItem(GGameTurn& trn, GLockData& data, TLockMode mode);

#endif
