/**
  *  \file game/mine.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_MINE_H
#define PCC_V2_GAME_MINE_H

#include <map>
#include "game/obj.h"
#include "cpluslib/smartptr.h"
#include "cpluslib/enum.h"
#include "cpluslib/types.h"

class GGameTurn;
class GMinefieldList;

//! A Minefield.
class GMinefield : public GCircularObject {
    // friend class GMinefieldList;
    
    int   id;
    int   x, y;
    int   owner;
    int   radius_now;           // computed from below
    int32 units_now;            // computed from below
    bool  web;

    int32 units;
    int   turn;
    int32 units_prev;
    int   turn_prev;
    int   last_action;
 public:
    GMinefield(int id, int ac = 0);

    void internalCheck(GGameTurn& trn);

    void setData(int x, int y, int own, int32 units, int turn);
    void setData(int x, int y, int own, int32 units, int turn, bool web);

    /* GMapObject interface */
    string_t getName() const;
    string_t getFullName() const;
    int      getId() const;
    int      getOwner() const;
    bool     isExistant() const;
    GPoint   getPos() const;
    int      getRadius() const;

    /* Minefield Interface */
    int32    getUnitsAfterDecay(int32 orig_units);

    int      getAction() const { return last_action; }
    void     setAction(int ac) { last_action = ac; }
    int32    getUnits() const { return units_now; }
    bool     isWeb() const { return web; }
    int      getTurn() const { return turn; }

    /** Actions that lead to Minefield Update. These must keep their
        relative and absolute values, because these equal the `ScanType'
        of a UTILx.DAT entry, plus one. */
    enum {
        MFA_NONE,               ///< No action, i.e. minefield not scanned this turn
        MFA_LAID,               ///< Minefield laid this turn.
        MFA_SWEPT,              ///< Minefield swept this turn.
        MFA_SCANNED             ///< Minefield scanned this turn.
    };
};

#endif
