/**
  *  \file game/msg.h
  *  \brief Subspace Message Handling
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_MSG_H
#define PCC_V2_GAME_MSG_H

#include <vector>

#include "cpluslib/smartptr.h"
#include "fileio/streams.h"
#include "cpluslib/sigslot.h"
#include "cpluslib/types.h"
#include "cpluslib/string.h"

//! Subspace message box.
class GMessagebox {
    int current;
 public:
    GMessagebox();
    virtual ~GMessagebox();

    virtual int      getCount() = 0;
    virtual string_t getText(int index) = 0;
    virtual string_t getHeading(int index) = 0;
    virtual int      getStatus(int index) = 0;
    virtual void     setStatus(int index, int newstat) = 0;

    int getCurrent() const { return current; }
    void setCurrent(int aindex);

    Signal0 mbox_changed;
};

//! Standard Inbox.
class GInbox : public GMessagebox {
    struct Message;
    std::vector<Message*> messages;

    void addMessage(string_t str);
 public:
    GInbox();
    ~GInbox();
    int      getCount();
    string_t getText(int index);
    string_t getHeading(int index);
    int      getStatus(int index);
    void     setStatus(int index, int newstat);

    void loadInbox(Ptr<Stream> str);
};

//! Standard Outbox.
class GOutbox : public GMessagebox {
    struct Message;
    std::vector<Message*> messages;
    void clear();
    int sender;
    bool is_winplan;
 public:
    GOutbox(int sender);
    ~GOutbox();

    int      getCount();
    string_t getText(int index);
    string_t getHeading(int index);
    int      getStatus(int index);
    void     setStatus(int index, int newstat);
    string_t getSendPrefix(int index, int to);
    string_t getRawText(int index);
    uint16   getReceiverMask(int index);
    int      getSender() const { return sender; }

    void     deleteMessagesAfter(int index);
    void     deleteMessage(int index);

    void     setFormat(bool is_wp) { is_winplan = is_wp; }
    bool     getFormat() const { return is_winplan; }

    void     addMessage(string_t text, uint16 receivers);
    void     addMessageFromFile(string_t text, uint16 receivers);

    /** All permitted receivers (Host + 11 players) */
    static const uint16 ALL_RECEIVERS_MASK = 0x1FFE;
    /** Receivers of a standard universal message (11 players) */
    static const uint16 UNIVERSAL_MASK     = 0x0FFE;
    static const char   UNIVERSAL_TEXT[];
};

string_t getMessageFromArray(const char* array, std::size_t size, bool rewrap);
void     encodeMessage(string_t& text);

#endif
