/**
  *  \file game/objtypes.h
  *  \brief Object Type
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_OBJTYPES_H
#define PCC_V2_GAME_OBJTYPES_H

#include "cpluslib/string.h"
#include "cpluslib/sigslot.h"

class GGameTurn;
class GObject;

class GObjectType {
    GObjectType(const GObjectType&);
    GObjectType& operator=(const GObjectType&);
 protected:
    GGameTurn& trn;
    int current;
    int count;
    int max;
 public:
    GObjectType(int amax, GGameTurn& atrn);
    virtual ~GObjectType();

    virtual void init();
    virtual bool isExistant(int index)    = 0;
    virtual bool isMarked(int index)      = 0;
    virtual void toggleMark(int index)    = 0;
    virtual int  getDirtyFlags(int index) = 0;
    virtual string_t getName(int index)   = 0;
    virtual string_t getClassName()       = 0;
    virtual GObject& getObject(int index) = 0;

    virtual int  getNext(int index, bool mark);
    virtual int  getPrev(int index, bool mark);

    virtual void setCurrent(int index);
    int          getCurrent() const { return current; }

    int          getCount() const { return count; }

    GGameTurn&   getTurn() const { return trn; }

    Signal1<int> sig_pos_changed;
};

#endif
