/**
  *  \file game/pvcr.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_PVCR_H
#define PCC_V2_GAME_PVCR_H

#include "game/vcr.h"

#define PVCR_INTEGER

class VcrPlayerPHost : public VcrPlayer {
 public:
    enum {
        VCR_MAX_BEAMS = 20,
        VCR_MAX_TORPS = 20,
        VCR_MAX_FTRS  = 50,
        VCR_MAX_BAYS  = 50      // 50 allowed on planets, 20 on ship
    };
 private:
    typedef double damage_t;
    /** Changing values. */
    struct RunningStatus {
        int  beam_status[VCR_MAX_BEAMS];        ///< Beam status, [0, ~1000]
        int  torp_status[VCR_MAX_TORPS];        ///< Torpedo launcher status, [0, ~1000]
        int  bay_status[VCR_MAX_BAYS];          ///< Fighter bay status, [0, ~1000]
        char fighter_status[VCR_MAX_FTRS];      ///< Fighter status, [0, 2]
        int  fighter_shots_left[VCR_MAX_FTRS];  ///< Fighter strikes remaining
        int  fighter_x[VCR_MAX_FTRS];           ///< Fighter X position, in meters
        int  obj_x;                             ///< X position of baseship, in meters
        int  launched_fighters;                 ///< Number of fighters currently out
        int  launch_countdown;                  ///< Countdown to next fighter launch
#ifndef PVCR_INTEGER
        damage_t shield;        ///< Shield status, [0, 100]
        damage_t damage;        ///< Damage, [0, 150]
        damage_t crew;          ///< Crew, [0, 100]
#else
        int32 shield_scaled;    ///< Shield status, scaled with scale
        int32 damage_scaled2;   ///< Damage, scaled with scale*100
        int32 crew_scaled2;     ///< Crew, scaled with scale*100
#endif
    };

    /** Precomputed values which do not change. */
    struct FixedStatus {
        int beam_recharge;      ///< Precomputed gross beam recharge rate.
        int beam_hit_odds;      ///< Precomputed gross beam hit odds (%)
        int torp_hit_odds;      ///< Precomputed gross torp hit odds (%)
        int torp_recharge;      ///< Precomputed torp recharge rate
        int bay_recharge;       ///< Precomputed bay recharge rate
#ifdef PVCR_INTEGER
        int32 scale;            ///< Scale factor for status value
        int32 max_scaled;       ///< Maximum damage value. 10000 * scale
        int32 damage_limit_scaled; ///< Scaled damage limit, scaled with 100*scale
#else
        int damage_limit;       ///< Damage limit (150 or 100).
#endif
        /// \name Cached and adjusted values of combat configuration options.
        //@{
        int ShieldDamageScaling;
        int ShieldKillScaling;  // new 4.0
        int HullDamageScaling;
        int CrewKillScaling;
        int MaxFightersLaunched;
        int StrikesPerFighter;
        int BayLaunchInterval;
        int FighterMovementSpeed;
        int FighterBeamExplosive;
        int FighterBeamKill;
        int FighterFiringRange;
        int BeamHitFighterRange;
        int BeamHitFighterCharge;
        int BeamFiringRange;
        int BeamHitShipCharge;
        int TorpFiringRange;
        int ShipMovementSpeed;
        ///@}
    };

    struct Status {
        FixedStatus f;
        RunningStatus r;
    };

    struct StatusToken : public VcrStatus {
        Status status[2];
        int seed;
        int status_word;
        bool done;
        int one_f, right_probab;
        TVcr vcr;
        StatusToken(time_type t);
    };

    VCR_STATIC TVcr vcr;
    VCR_STATIC Status status[2];
    VCR_STATIC uint16 seed;
    VCR_STATIC bool done;
    VCR_STATIC int one_f, right_probab;
    VCR_STATIC unsigned capabilities;

    /** Ruleset selection. True if PHost 3 or later. */
    bool phost3;

    VCR_STATIC inline uint16 random64k();
    VCR_STATIC inline int randomRange(int n);

    VCR_STATIC inline bool isPlanet(int side);

#ifdef PVCR_INTEGER
    VCR_STATIC inline int32 computeShieldDamageS(int expl, int kill, int mass, int side);
    VCR_STATIC inline int32 computeHullDamageS(int expl, int kill, int mass, int side);
    VCR_STATIC inline int32 computeCrewKilledS(int kill, int mass, bool death, int side);
#else
    VCR_STATIC inline damage_t computeShieldDamage(int expl, int kill, int mass, int side);
    VCR_STATIC inline damage_t computeHullDamage(int expl, int kill, int mass, int side);
    VCR_STATIC inline damage_t computeCrewKilled(int kill, int mass, bool death, int side);
#endif
    VCR_STATIC bool hit(int side, int kill, int expl, bool is_death_ray);

    VCR_STATIC inline int computeBayRechargeRate(int num, int side, int exp);
    VCR_STATIC void fighterRecharge(int side);
    VCR_STATIC void fighterLaunch(int side);
    VCR_STATIC void fighterMove(int side);
    VCR_STATIC void fighterIntercept();
    VCR_STATIC bool fighterAttack(int side);

    VCR_STATIC inline int computeBeamHitOdds(int beam, int side, int exp);
    VCR_STATIC inline int computeBeamRechargeRate(int beam, int side, int exp);
    VCR_STATIC void beamRecharge(int side);
    VCR_STATIC int beamFindNearestFighter(int side);
    VCR_STATIC bool beamFire(int side);

    VCR_STATIC inline int computeTorpHitOdds(int torp, int side, int exp);
    VCR_STATIC inline int computeTubeRechargeRate(int torp, int side, int exp);
    VCR_STATIC void torpsRecharge(int side);
    VCR_STATIC bool torpsFire(int side);

    VCR_STATIC void moveObjects();
    VCR_STATIC bool canStillFight(int side);

    bool checkVcrSide(TVcr& vcr, int side);
 public:
    VcrPlayerPHost(bool av3);
    bool checkVcr(TVcr& vcr);
    void initVcr(TVcr& vcr);
    void doneVcr(TVcr& vcr);
    bool setCapabilities(uint16 cap);
    bool playCycle();
    void playFastForward();

    int getBeamStatus(int side, int id);
    int getTorpStatus(int side, int id);
    int getTorpCount(int side);
    int getFighterCount(int side);
    int getShields(int side);
    int getDamage(int side);
    int getCrew(int side);

    int getFighterX(int side, int id);
    int getFighterStatus(int side, int id);
    int getObjectX(int side);
    long getDistance();

    string_t getPlayerName() const;

    Ptr<VcrStatus> getStatusToken();
    void setStatus(Ptr<VcrStatus> status);
};

#endif
