/**
  *  \file game/rst.h
  *  \brief Result File Parsing
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_RST_H
#define PCC_V2_GAME_RST_H

#include "fileio/streams.h"
#include "game/struct.h"

//! Result File Parser.
class GResultFile {
    TResultHeader header;
    Ptr<Stream>   file;
    int           version;

    void parseWinplanVersion();
 public:
    GResultFile(Ptr<Stream> afile);
    ~GResultFile();

    int getVersion() const { return version; }
    int32 getSectionOffset(int section) const;
    Ptr<Stream> getFile() const { return file; }

    /** RST Section numbers. */
    enum {
        SEC_SHIPS,              ///< Player's ships.
        SEC_TARGETS,            ///< Visual contacts.
        SEC_PLANETS,            ///< Player's planets.
        SEC_BASES,              ///< Player's bases.
        SEC_MESSAGES,           ///< Incoming messages.
        SEC_SHIPXY,             ///< Ship X/Y, nonvisual contacts.
        SEC_GEN,                ///< Red tape.
        SEC_VCRS,               ///< Combat recordings.
        SEC_KORE,               ///< "KORE" (minefields, race names, storms, contacts, UFOs).
        SEC_LEECH,              ///< "LEECH.DAT".
        SEC_SKORE               ///< "SKORE" (more UFOs).
    };
};

#endif
