/**
  *  \file tvcr.h
  *  \brief THost VCR class
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_TVCR_H
#define PCC_V2_TVCR_H

#include "game/vcr.h"

class VcrPlayerTHost : public VcrPlayer {
 public:
    enum {
        VCR_MAX_BEAMS = 10,
        VCR_MAX_TORPS = 10,
        VCR_MAX_FTRS  = 19      /* max # of tracks */
    };
    typedef int rand_type;
    typedef unsigned char rand_table_type;
 private:
    struct Status {
        int beam_status[VCR_MAX_BEAMS];
        int torp_status[VCR_MAX_TORPS];
        int ftr_active[VCR_MAX_FTRS];
        int ftr_x[VCR_MAX_FTRS];
        int cur_x;
        int damage_limit;
        void clear();
    };
    struct StatusToken : public VcrStatus {
        Status status[2];
        int seed;
        int status_word;
        TVcr vcr;
        StatusToken(time_type t);
    };    
    
    VCR_STATIC TVcr vcr;
    VCR_STATIC Status status[2];
    VCR_STATIC int seed;

    VCR_STATIC inline rand_type psrandom_1_20();
    VCR_STATIC inline rand_type psrandom_1_100();
    VCR_STATIC inline rand_type psrandom_1_17();
    
    VCR_STATIC inline bool isPlanet(int side);
    VCR_STATIC inline bool isFreighter(int side);

    VCR_STATIC void hit(int side, int damage, int kill);

    VCR_STATIC inline void launchFighter(int side);
    VCR_STATIC void launchFighters(int side);
    VCR_STATIC void moveFighters();
    VCR_STATIC inline void fighterShootLeft(int i);
    VCR_STATIC inline void fighterShootRight(int i);
    VCR_STATIC inline void killFighter(int i, int side);
    VCR_STATIC void fighterStuff();

    VCR_STATIC void rechargeBeams(int side);
    VCR_STATIC void fireBeam(int from, int which);
    VCR_STATIC void fireBeams(int from);
    VCR_STATIC void fireAtFighter(int from, int beam);
    VCR_STATIC void fireBeamsAtFighter(int fm);

    VCR_STATIC inline void fireTorp(int from, int launcher);
    VCR_STATIC void fireTorpedoes(int from);

    VCR_STATIC void preloadWeapons(int side);

    static bool checkVcrSide(TVcr& avcr, int side);
 public:
    bool checkVcr(TVcr& avcr);
    void initVcr(TVcr& avcr);
    void doneVcr(TVcr& avcr);
    bool setCapabilities(uint16 cap);

    bool playCycle();
    void playFastForward();

    int getBeamStatus(int side, int id);
    int getTorpStatus(int side, int id);
    int getTorpCount(int side);
    int getFighterCount(int side);
    int getShields(int side);
    int getDamage(int side);
    int getCrew(int side);
    int getFighterX(int side, int id);
    int getFighterStatus(int side, int id);
    int getObjectX(int side);
    long getDistance();

    Ptr<VcrStatus> getStatusToken();
    void setStatus(Ptr<VcrStatus> status);

    string_t getPlayerName() const;
};

#endif
