/**
  *  \file gfx/anim.h
  *  \brief Animations
  *
  *  Data structures and classes for animated sprites.
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GFX_ANIM_H
#define PCC_V2_GFX_ANIM_H

#include <vector>
#include "gfx/pixmap.h"
#include "gfx/sprite.h"
#include "cpluslib/assert.h"
#include "cpluslib/smartptr.h"

struct GfxAniFrame {
    Ptr<GfxPixmap> pixmap;      /**< Pixmap for this frame. */
    int next;                   /**< Index into array, for next frame. negative at end. */
    int time;                   /**< Time this frame is displayed, in ticks. */

    GfxAniFrame(Ptr<GfxPixmap> apix, int anext, int atime)
        : pixmap(apix), next(anext), time(atime) { }
};

class GfxAnimation {
    std::vector<GfxAniFrame> data;
    int wi, he;
 public:
    GfxAnimation();
    ~GfxAnimation();
    void addFrame(Ptr<GfxPixmap> apix, int anext, int atime);
    GfxAniFrame& getFrame(int i) {
        ASSERT (i >= 0 && unsigned(i) < data.size());
        return data[i];
    }
    int getStart() const { return 0; }
    int getWidth() const { return wi; }
    int getHeight() const { return he; }
    int getNumFrames() const { return data.size(); }
};

class GfxAniSprite : public GfxSprite {
    Ptr<GfxAnimation> anim;
    int ip;                     /**< "instruction pointer". */
    int time;                   /**< Remaining time for current frame. */
    void setIndex(int aip);
 public:
    GfxAniSprite(int x, int y, int alayer, Ptr<GfxAnimation> aanim);
    void tick();
    void draw(Ptr<GfxCanvas> can);
};

#endif
