/**
  *  \file gfx/clip.h
  *  \brief Simple Clipping
  *
  *  Classes for clipping output at a single rectangle.
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GFX_CLIP_H
#define PCC_V2_GFX_CLIP_H

#include "gfx/gfx.h"

class GfxClipRect : public GfxFilter {
 protected:
    GfxRect rect;
    GfxClipRect(const GfxRect& arect, Ptr<GfxCanvas> apar = 0);
 public:
    void drawHLine(int x1, int y1, int x2, uint32 color, uint8 pat, alpha_type alpha);
    void drawVLine(int x1, int y1, int y2, uint32 color, uint8 pat, alpha_type alpha);
    void drawPixel(int x, int y, uint32 color, alpha_type alpha);
    void drawBar(GfxRect rect, uint32 color, uint32 bg, const GfxFillPattern& pat, alpha_type alpha);

    void blitSurface(int x, int y, SDL_Surface* src, GfxRect rect);
    void blitPattern(GfxRect rect, int x, int y, int bytes_per_line, const uint8* data, uint32 color, uint32 bg, alpha_type alpha);

    GfxRect computeClipRect(GfxRect r);
};

class GfxClipRectFilter : public GfxClipRect {
 public:
    GfxClipRectFilter(const GfxRect& arect, Ptr<GfxCanvas> apar = 0);
    void setClipRect(const GfxRect& arect) { rect = arect; }
};

class GfxOffset : public GfxFilter {
 protected:
    int dx, dy;
 public:
    GfxOffset(int adx, int ady, Ptr<GfxCanvas> apar = 0);
    
    void drawHLine(int x1, int y1, int x2, uint32 color, uint8 pat, alpha_type alpha);
    void drawVLine(int x1, int y1, int y2, uint32 color, uint8 pat, alpha_type alpha);
    void drawPixel(int x, int y, uint32 color, alpha_type alpha);
    void drawBar(GfxRect rect, uint32 color, uint32 bg, const GfxFillPattern& pat, alpha_type alpha);

    void blitSurface(int x, int y, SDL_Surface* src, GfxRect rect);
    void blitPattern(GfxRect rect, int x, int y, int bytes_per_line, const uint8* data, uint32 color, uint32 bg, alpha_type alpha);

    GfxRect computeClipRect(GfxRect r);
    void update(GfxRect r);

    uint32 getPixel(int x, int y);
};

#endif
