/**
  *  \file gfx/font.h
  *  \brief Font Driver Interface
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GFX_FONT_H
#define PCC_V2_GFX_FONT_H

#include <string>
#include "gfx/gfx.h"
#include "fileio/streams.h"

class GfxFont : public Object {
 public:
    /** Font driver capabilities. */
    enum {
        FN_LINEAR   = 1,        /**< Linear, i.e. no kerning, huge descends, etc. */
        FN_SCALABLE = 2,        /**< Scalable, i.e. setScale() works */
        FN_ALPHA    = 4         /**< Alpha, i.e. outText() uses \c parm.alpha */
    };

    /** Output text using the specified parameters.
        \param out    output canvas
        \param parm   graphics parameters. outText() should use at least
                      color, bgcolor, and alpha; and it should \em not
                      use just_x/just_y.
        \param x,y    coordinates (top-left pixel)
        \param text   text */
    virtual void outText(GfxCanvas* out,
                         GfxParameters& parm,
                         int x, int y,
                         string_t text) = 0;
    virtual int getTextWidth(string_t text) = 0;
    virtual int getTextHeight(string_t text) = 0;

    virtual bool setScale(int mul, int div) = 0;
    virtual GfxPoint getScale() = 0;

    virtual unsigned getCapabilities() = 0;
    
};

class GfxStdFont : public GfxFont {
    ArrayPtr<char> image;       /**< Image of font file */
    std::size_t base_ptr;       /**< Offset of font in file */
    bool antialias;             /**< "Antialiasing" required? */

    const char* getChar(char c);
    void initialize(int aindex);
 public:
    GfxStdFont(Ptr<Stream> stream, int aindex = 0);
    GfxStdFont(ArrayPtr<char> aimage, int aindex = 0);

    void outText(GfxCanvas* out, GfxParameters& parm, int x, int y, string_t text);
    int getTextWidth(string_t text);
    int getTextHeight(string_t text);
    
    bool setScale(int mul, int div);
    GfxPoint getScale();

    unsigned getCapabilities();
};

void outText(GfxCanvas* out, GfxParameters& parm, GfxFont* fnt, int x, int y, string_t text);
void outTextF(GfxCanvas* out, GfxParameters& parm, GfxFont* fnt, int x, int y, int maxw, string_t text);

#endif
