/**
  *  \file gfx/gfxalpha.cc
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "gfx/gfxalpha.h"

GfxAlphaFilter::GfxAlphaFilter(uint8 f_alpha, Ptr<GfxCanvas> can)
    : GfxFilter(can), f_alpha(f_alpha)
{ }

GfxAlphaFilter::~GfxAlphaFilter()
{ }

static inline int
multAlpha(uint8 a, uint8 b)
{
    return (a * b) / 255;
}

void
GfxAlphaFilter::drawHLine(int x1, int y1, int x2, uint32 color, uint8 pat, alpha_type alpha)
{
    parent->drawHLine(x1, y1, x2, color, pat, multAlpha(alpha, f_alpha));
}

void
GfxAlphaFilter::drawVLine(int x1, int y1, int y2, uint32 color, uint8 pat, alpha_type alpha)
{
    parent->drawVLine(x1, y1, y2, color, pat, multAlpha(alpha, f_alpha));
}

void
GfxAlphaFilter::drawPixel(int x, int y, uint32 color, alpha_type alpha)
{
    parent->drawPixel(x, y, color, multAlpha(alpha, f_alpha));
}

void
GfxAlphaFilter::drawBar(GfxRect rect, uint32 color, uint32 bg, const GfxFillPattern& pat, alpha_type alpha)
{
    parent->drawBar(rect, color, bg, pat, multAlpha(alpha, f_alpha));
}

void
GfxAlphaFilter::blitSurface(int x, int y, SDL_Surface* src, GfxRect rect)
{
    // FIXME: alpha
    parent->blitSurface(x, y, src, rect);
}

void
GfxAlphaFilter::blitPattern(GfxRect rect, int x, int y, int bytes_per_line, const uint8* data, uint32 color, uint32 bg, alpha_type alpha)
{
    parent->blitPattern(rect, x, y, bytes_per_line, data, color, bg, multAlpha(alpha, f_alpha));
}

GfxRect
GfxAlphaFilter::computeClipRect(GfxRect r)
{
    return parent->computeClipRect(r);
}

bool
GfxAlphaFilter::isVisible(GfxRect r)
{
    return parent->isVisible(r);
}

bool
GfxAlphaFilter::isClipped(GfxRect r)
{
    return parent->isClipped(r);
}
