/**
  *  \file palette.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_PALETTE_H
#define PCC_V2_PALETTE_H

#include "defs.h"
#include "cpluslib/types.h"
#ifdef HAVE_SDL
#  include <SDL_video.h>
#endif

enum { NUM_STANDARD_COLORS = 256 };

/** Symbolic Names for Standard Colors. The palette is "historically grown"
    from PCC v1. PCC v1 replicated slots 0 to 15 as 16..31 and 32..47,
    to simplify display of multiple 16-color pictures. We do no longer do
    that, so we have 96 free color slots available for picture mixing, not
    just 64.

    Note that the image loading code in pixmap.cc depends on the palette
    contents (and the images, too, of course). */
enum {
    /* PCC v1 16-color palette */
    COLOR_BLACK      = 0,
    COLOR_DARK       = 1,
    COLOR_GRAY       = 2,
    COLOR_GREEN      = 3,
    COLOR_RED        = 4,
    COLOR_GREENBLACK = 5,
    COLOR_BLUE       = 6,
    COLOR_BLUEGRAY   = 7,
    COLOR_BLUEBLACK  = 8,
    COLOR_YELLOW     = 9,
    COLOR_VCR3       = 10,       ///< 3 colors for VCR. Also used by 16-color images in 16-color mode in PCC v1
    COLOR_UNUSED2    = 13,       ///< 2 slots unused here. Used by 16-color images in 16-color mode in PCC v1
    COLOR_WHITE      = 15,

    /* PCC v1 256-color palette, moved by -32 slots */
    COLOR_GRAYSCALE     = 16,    ///< Grayscales. (COLOR_BLACK, COLOR_GRAYSCALE, ... COLOR_GRAYSCALE + 15)
    COLOR_FIRE          = 32,    ///< Fire colors. black/red/yellow/white gradient (COLOR_FIRE, ..., COLOR_FIRE + 31, COLOR_WHITE)
    COLOR_SHIELD        = 64,    ///< Shield colors. black/blue/white gradient (COLOR_SHIELD, ..., COLOR_SHIELD + 15)
    COLOR_STATUS        = 80,    ///< Status bar colors. red/green gradient (COLOR_STATUS, ..., COLOR_STATUS + 15)

    COLOR_BLUEBERRYICE  = 96,    ///< Blueberry ice/yoghurt. How'd you call that?

    /* the following colors are available as 11 to 30 in CCScript,
       and they are used to map UFO colors. */
    COLOR_DARKBLUE         = 97,
    COLOR_BRIGHTBLUE       = 98,
    COLOR_DARKGREEN        = 99,
    COLOR_BRIGHTGREEN      = 100,
    COLOR_DARKCYAN         = 101,
    COLOR_BRIGHTCYAN       = 102,
    COLOR_DARKRED          = 103,
    COLOR_BRIGHTRED        = 104,
    COLOR_DARKMAGENTA      = 105,
    COLOR_BRIGHTMAGENTA    = 106,
    COLOR_DARKYELLOW       = 107, ///< pretty dirty-looking yellow
    COLOR_BRIGHTYELLOW     = 108,
    COLOR_BROWN            = 109, ///< greenish brown
    COLOR_BRIGHTBROWN      = 110, ///< yellowish brown
    COLOR_PINK             = 111,
    COLOR_BRIGHTPINK       = 112, ///< body-color pink
    COLOR_ORANGE           = 113,
    COLOR_BRIGHTORANGE     = 114, ///< yellowish orange
    COLOR_BLUEBERRY        = 115,
    COLOR_BLUEPINK         = 116, ///< blue'ish pink

    /* Miscellaneous */
    COLOR_GREENSCALE6      = 117, ///< 6 scales of green (COLOR_GREENBLACK, COLOR_GREENSCALE, ... COLOR_GREENSCALE + 5, COLOR_GREEN)
    COLOR_DARKPINK         = 123, ///< pretty dark pink
    COLOR_DARKYELLOWSCALE3 = 124, ///< 3 very dark yellow scales; used in the title backdrop (COLOR_DARKYELLOWSCALE3, ... COLOR_DARKYELLOWSCALE3 + 2)
    COLOR_DARKPINK2        = 127, ///< not so dark pink
    COLOR_GREENSCALE       = 128, ///< 16 scales of green (COLOR_BLACK, COLOR_GREENSCALE, ... COLOR_GREENSCALE + 15)
    COLOR_DARKYELLOWSCALE  = 144, ///< 8 scales of yellow (COLOR_BLACK, COLOR_DARKYELLOWSCALE, ... COLOR_DARKYELLOWSCALE + 7)
    
    COLOR_UNUSED8          = 152, ///< 8 slots unused

    /* colors available for image mixing */
    COLOR_AVAIL            = 160
};

#ifdef HAVE_SDL
extern SDL_Color standard_palette[NUM_STANDARD_COLORS];
extern uint32    standard_colors[NUM_STANDARD_COLORS];

void initPalette(SDL_Surface* sfc);
#endif

#endif
