/**
  *  \file gfx/pixmap.h
  *  \brief Pixmaps
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GFX_PIXMAP_H
#define PCC_V2_GFX_PIXMAP_H

#include "gfx/gfx.h"
#include "fileio/streams.h"

class GfxPixmap : public GfxSurface {
 public:
    GfxPixmap(int w, int h, int depth, uint32 rmask, uint32 gmask, uint32 bmask, uint32 amask);
    GfxPixmap(int w, int h, SDL_PixelFormat* format);
    GfxPixmap(SDL_Surface* sfc, bool aown);

    void setFormat(int w, int h, SDL_PixelFormat* format);
    void setFormat(int w, int h, int depth, uint32 rmask, uint32 gmask, uint32 bmask, uint32 amask);

    Ptr<GfxPixmap> convertTo(SDL_PixelFormat* format);
    Ptr<GfxPixmap> convertToScreenFormat();

    void blit(GfxCanvas* can, int x, int y);
    void blit(GfxCanvas* can, int x, int y, GfxRect subr);
    void blitSized(GfxCanvas* can, int x, int y, int wi, int he);

    void tile(GfxCanvas* can, const GfxRect& area, int alt=0);
    void tileAnchored(GfxCanvas* can, const GfxRect& area, int ax=0, int ay=0, int alt=0);

    void flipHorizontal();
    void flipVertical();

    int getWidth() const { return sfc->w; }
    int getHeight() const { return sfc->h; }

    Ptr<GfxPixmap> clone();
};

Ptr<GfxPixmap> loadImage(Ptr<Stream> s);

#endif
