/**
  *  \file gfx/sprite.h
  *  \brief Sprites
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GFX_SPRITE_H
#define PCC_V2_GFX_SPRITE_H

#include <vector>
#include "gfx/gfxutil.h"
#include "gfx/gfx.h"
#include "gfx/pixmap.h"

class GfxSpriteManagerData;
class GfxSpriteManager;

//! Sprite.
class GfxSprite : public Object {
    friend class GfxSpriteManager;
    Ptr<GfxSpriteManagerData> mgr;
    Ptr<GfxSprite> next, prev;
    GfxSpriteManager* manager;
 protected:
    GfxRect pos;
    int layer;
 public:
    GfxSprite(const GfxRect& apos, int alayer);
    ~GfxSprite();
    void moveTo(int x, int y);
    void moveToC(int x, int y);
    void update();
    void deleteThis();
    virtual void tick();
    virtual void draw(Ptr<GfxCanvas> can) = 0;

    void setManager(GfxSpriteManager* amanager);
    GfxSpriteManager* getManager() const { return manager; }

    const GfxRect& getPos() const { return pos; }
    int getPosCX() const { return pos.x + pos.w / 2; }
    int getPosCY() const { return pos.y + pos.h / 2; }
    int getHeight() const { return pos.h; }
    int getWidth() const { return pos.w; }
    int getLayer() const { return layer; }
    GfxSpriteManagerData* getManagerData() const { return mgr; }
};

//! Sprite Manager (container).
class GfxSpriteManager {
    std::vector<Ptr<GfxSprite> > sprite_list;
    GfxRect bbox;
 public:
    GfxSpriteManager();
    ~GfxSpriteManager();
    
    void addSprite(Ptr<GfxSprite> asprite);
    void tick(Ptr<GfxCanvas> can);
    void tickSkip(bool call_tick = true);

    void moveSprite(Ptr<GfxSprite> asprite, int ax, int ay);
    void deleteSprite(Ptr<GfxSprite> asprite);
    void deleteLayer(int layer);

    void draw(Ptr<GfxCanvas> can);

    void drawVoid(Ptr<GfxCanvas> can, GfxRect r);

    void setBoundingBox(const GfxRect& r) { bbox = r; }
    const GfxRect& getBoundingBox() const { return bbox; }

    bool isLayerEmpty(int layer) const;
};

class GfxPixmapSprite : public GfxSprite {
    Ptr<GfxPixmap> pix;
 public:
    GfxPixmapSprite(int x, int y, int l, Ptr<GfxPixmap> apix);
    GfxPixmapSprite(int x, int y, int l, Ptr<GfxPixmap> apix, bool);
    void draw(Ptr<GfxCanvas> can);
};

#endif
