/**
  *  \file int/expr.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_EXPR_H
#define PCC_V2_INT_EXPR_H

#include <list>
#include <cstddef>
#include "int/data.h"
#include "int/lex.h"
#include "int/fun-tbl.h"

class IntExprNode;
class IntByteCodeObject;

typedef Ptr<IntExprNode> IntExprNodePtr;
//typedef IntExprNode* IntExprNodePtr;

struct IntExecutionContext {
    Ptr<IntContext> context;
};

class IntExprNode : public Object {
 public:
    IntExprNode();
    ~IntExprNode();

    virtual IntObject eval(IntExecutionContext& con) = 0;
    
    virtual void compileV(IntByteCodeObject&) = 0;
    virtual void compileS(IntByteCodeObject&) = 0;
    virtual void compileA(IntByteCodeObject&) = 0;

    static double checkFloatVal(double);
};
#define RETURN_FLOAT_VAL(d) return IntExprNode::checkFloatVal(d)

class IntExprLValue : public IntExprNode {
 protected:
    IntExprLValue() { }
 public:
    void compileA(IntByteCodeObject&);
};

class IntExprFuncall1 : public IntExprLValue {
    IntUnaryFunction func;
    IntExprNodePtr  a;
 public:
    IntExprFuncall1(IntUnaryFunction, IntExprNodePtr);
    ~IntExprFuncall1();

    IntObject eval(IntExecutionContext& con);
    void compileV(IntByteCodeObject&);
    void compileS(IntByteCodeObject&);
};

class IntExprFuncall2 : public IntExprLValue {
    IntBinaryFunction func;
    IntExprNodePtr  a;
    IntExprNodePtr  b;
 public:
    IntExprFuncall2(IntBinaryFunction, IntExprNodePtr, IntExprNodePtr);
    ~IntExprFuncall2();

    IntObject eval(IntExecutionContext& con);
    void compileV(IntByteCodeObject&);
    void compileS(IntByteCodeObject&);
};

class IntExprLiteral : public IntExprLValue {
    IntObject value;
 public:
    IntExprLiteral(const IntObject& v);
    ~IntExprLiteral();
    IntObject eval(IntExecutionContext& con);
    void compileV(IntByteCodeObject&);
    void compileS(IntByteCodeObject&);
};

class IntExprFileHandle : public IntExprLValue {
    IntExprNodePtr a;
 public:
    IntExprFileHandle(IntExprNodePtr aa);
    ~IntExprFileHandle();
    IntObject eval(IntExecutionContext& con);
    void compileV(IntByteCodeObject&);
    void compileS(IntByteCodeObject&);
};

class IntExprParser {
    std::list<IntExprNodePtr> stack;
    IntLexer& lex;
    void parse0();
    void parse0a();
    void parse0b();
    void parse1();
    void parse2();
    void parse3();
    void parse4();
    void parse5();
    void parse6();
    void parse7();
    void parse8();
    void parse9();
    void parse10();
 public:
    IntExprParser(IntLexer& l);
    ~IntExprParser();

    void           parseExpr() { parse0(); }
    void           push(IntExprNodePtr n);
    void           pushFuncall(IntUnaryFunction);
    void           pushFuncall(IntBinaryFunction);
    IntExprNodePtr pop();
    std::size_t    parseArglist();
};

#endif
