/**
  *  \file int/lex.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_LEX_H
#define PCC_V2_INT_LEX_H

#include "util/except.h"
#include "cpluslib/string.h"
#include "cpluslib/types.h"

#undef PI                       // this sucks

enum IntTokenType {
    /* These must equal the res_XXX from data.h */
    tt_Null,                   ///< "empty" (=EOL).
    tt_Bool,                   ///< boolean.
    tt_Int,                    ///< integer.
    tt_Float,                  ///< float.
    tt_String,                 ///< string.

    tt_Id,
    tt_Invalid,
    tt_NE, tt_GE, tt_LE, tt_Asgn,
    tt_AND, tt_OR, tt_XOR, tt_NOT, tt_MOD,
    tt_Join,                    // &
    tt_Hash,                    // #
    tt_Plus,                    // +
    tt_Minus,                   // -
    tt_Mul,                     // *
    tt_RealDiv,                 // /
    tt_Div,                     // "\"
    tt_Pot,                     // ^
    tt_Left,                    // (
    tt_Right,                   // )
    tt_Comma,                   // ,
    tt_EQ,                      // =
    tt_LT,                      // <
    tt_GT,                      // >
    tt_Colon,                   // :
    tt_Semi,                    // ;
    tt_Dot                      // .
};

class IntLexer {
    string_t             the_line;
    string_t::size_type  index;
    IntTokenType         current_token;
    string_t             current_str;
    double               current_dbl;
    int32                current_int;

    inline char current() const;
 public:
    IntLexer(string_t line);

    const string_t& getString() const { return current_str; }
    double          getFloat()  const { return current_dbl; }
    int32           getInt()    const { return current_int; }
    IntTokenType    getType()   const { return current_token; }

    IntTokenType lex();
    
    bool lookingAt(IntTokenType t) {
        if (current_token == t) {
            lex();
            return true;
        } else
            return false;
    }

    static const double PI;
};

class IntException : public std::runtime_error {
 public:
    IntException(const string_t& awhat);
    IntException(const char* awhat)
        : std::runtime_error(awhat) { }
};

#endif
