/**
  *  \file game/pred.cc
  *  \brief Predicates for Matching Objects
  *
  *  These are used for matching objects. Instead of passing a function
  *  pointer or templatizing on a predicate, pass a `const GObjPredicate&'.
  *  This hierarchy is contravariant to the `GObject' hierarchy:
  *  because each `GMapObject' is a `GObject', each `GObject' predicate
  *  is also a `GMapObject' predicate.
  */

#include "game/pred.h"

GCirObjPredicate::~GCirObjPredicate()
{ }

GMapObjPredicate::~GMapObjPredicate()
{ }

bool
GMapObjPredicate::accept(GCircularObject* c) const
{
    return accept(static_cast<GMapObject*>(c));
}

GObjPredicate::~GObjPredicate()
{ }

bool
GObjPredicate::accept(GMapObject* c) const
{
    return accept(static_cast<GObject*>(c));
}

bool GMapObjIsAt::accept(GObject* obj) const
{
    return obj->getPos() == pt;
}

bool GObjAny::accept(GObject*) const
{
    return true;
}
