/**
  *  \file game/pvcr.h
  *
  *  (c) 2001 Stefan Reuther <Streu@gmx.de>
  */
#ifndef PCC_V2_GAME_PVCR_H
#define PCC_V2_GAME_PVCR_H

#include "game/vcr.h"

#define PVCR_INTEGER
#define PVCR_TEMPLATE template<int side>
#define PVCR_TPLCALL  <side>

class VcrPlayerPHost : public VcrPlayer {
 public:
    enum {
        VCR_MAX_BEAMS = 20,
        VCR_MAX_TORPS = 20,
        VCR_MAX_FTRS  = 50,
        VCR_MAX_BAYS  = 50      // 50 allowed on planets, 20 on ship
    };
 private:
    typedef double damage_t;
    /** Changing values. */
    struct RunningStatus {
        int beam_status[VCR_MAX_BEAMS];
        int torp_status[VCR_MAX_TORPS];
        int bay_status[VCR_MAX_BAYS];
        char fighter_status[VCR_MAX_FTRS];
        int fighter_shots_left[VCR_MAX_FTRS];
        int fighter_x[VCR_MAX_FTRS];
        int obj_x;
        int launched_fighters;
        int launch_countdown;
#ifndef PVCR_INTEGER
        damage_t shield;
        damage_t damage;
        damage_t crew;
#else
        int32 shield_scaled;    // scaled with scale
        int32 damage_scaled2;   // scaled with scale*100
        int32 crew_scaled2;     // scaled with scale*100
#endif
    };
    
    /** Precomputed values which do not change. */
    struct FixedStatus {
        int beam_recharge;
        int beam_hit_odds;
        int torp_hit_odds;
        int torp_recharge;
        int bay_recharge;
#ifdef PVCR_INTEGER
        int32 scale;
        int32 max_scaled;
        int32 damage_limit_scaled;
#else
        int damage_limit;
#endif
        int ShieldDamageScaling;
        int HullDamageScaling;
        int CrewKillScaling;
        int MaxFightersLaunched;
        int StrikesPerFighter;
        int BayLaunchInterval;
        int FighterMovementSpeed;
        int FighterBeamExplosive;
        int FighterBeamKill;
        int FighterFiringRange;
        int BeamHitFighterRange;
        int BeamHitFighterCharge;
        int BeamFiringRange;
        int BeamHitShipCharge;
        int TorpFiringRange;
        int ShipMovementSpeed;
    };

    struct Status {
        FixedStatus f;
        RunningStatus r;
    };
    
    struct StatusToken : public VcrStatus {
        RunningStatus status[2];
        int seed;
        int status_word;
        TVcr vcr;
        StatusToken(time_type t);
    };

    VCR_STATIC TVcr vcr;
//     VCR_STATIC RunningStatus rstatus[2];
//     VCR_STATIC FixedStatus fstatus[2];
    VCR_STATIC Status status[2];
    VCR_STATIC uint16 seed;
    VCR_STATIC bool done;
    VCR_STATIC int one_f, right_probab;

    /** true if PHost 3 rules, false for PHost 2. This one must be both
        static and nonstatic, to allow co-existence of a PHost3 and
        a PHost2 player even if PCC_STATIC_VCR is set. */
    bool phost3;

    VCR_STATIC inline uint16 random64k();
    VCR_STATIC inline int randomRange(int n);

    PVCR_TEMPLATE VCR_STATIC inline bool isPlanet();

#ifdef PVCR_INTEGER
    PVCR_TEMPLATE VCR_STATIC inline int32 computeShieldDamageS(int expl, int mass);
    PVCR_TEMPLATE VCR_STATIC inline int32 computeHullDamageS(int expl, int mass);
    PVCR_TEMPLATE VCR_STATIC inline int32 computeCrewKilledS(int kill, int mass);
#else
    PVCR_TEMPLATE VCR_STATIC inline damage_t computeShieldDamage(int expl, int mass);
    PVCR_TEMPLATE VCR_STATIC inline damage_t computeHullDamage(int expl, int mass);
    PVCR_TEMPLATE VCR_STATIC inline damage_t computeCrewKilled(int kill, int mass);
#endif
    PVCR_TEMPLATE VCR_STATIC bool hit(int kill, int expl);

    VCR_STATIC inline int computeBayRechargeRate(int num, int side);
    PVCR_TEMPLATE VCR_STATIC void fighterRecharge();
    PVCR_TEMPLATE VCR_STATIC void fighterLaunch();
    PVCR_TEMPLATE VCR_STATIC void fighterMove();
    VCR_STATIC void fighterIntercept();
    PVCR_TEMPLATE VCR_STATIC bool fighterAttack();
    
    VCR_STATIC inline int computeBeamHitOdds(int beam, int side);
    VCR_STATIC inline int computeBeamRechargeRate(int beam, int side);
    PVCR_TEMPLATE VCR_STATIC void beamRecharge();
    PVCR_TEMPLATE VCR_STATIC int beamFindNearestFighter();
    PVCR_TEMPLATE VCR_STATIC bool beamFire();

    VCR_STATIC inline int computeTorpHitOdds(int torp, int side);
    VCR_STATIC inline int computeTubeRechargeRate(int torp, int side);
    PVCR_TEMPLATE VCR_STATIC void torpsRecharge();
    PVCR_TEMPLATE VCR_STATIC bool torpsFire();

    VCR_STATIC void moveObjects();
    VCR_STATIC bool canStillFight(int side);

    bool checkVcrSide(TVcr& vcr, int side);
 public:
    VcrPlayerPHost(bool av3);
    bool checkVcr(TVcr& vcr);
    void initVcr(TVcr& vcr);
    void doneVcr(TVcr& vcr);
    bool playCycle();
    void playFastForward();

    int getBeamStatus(int side, int id);
    int getTorpStatus(int side, int id);
    int getTorpCount(int side);
    int getFighterCount(int side);
    int getShields(int side);
    int getDamage(int side);
    int getCrew(int side);

    int getFighterX(int side, int id);
    int getFighterStatus(int side, int id);
    int getObjectX(int side);

    string_t getPlayerName() const;

    Ptr<VcrStatus> getStatusToken();
    void setStatus(Ptr<VcrStatus> status);
};

#endif
