/*
 *  \file picedit/blit.inc
 *  \brief Scaled blit subroutines for editor
 */

#include "gfx/gfx-modetr.h"

template<class MT>
void
blit2(SDL_Surface* to, SDL_Surface* from, GfxRect target)
{
    /* clip is in pixmap coordinates */
    target.clipSurface(to);
    
    int lines = target.h / 2;
    int cols = target.w / 2;
    if (lines > from->h)
        lines = from->h;
    if (cols > from->w)
        cols = from->w;

    for (int y = 0; y < lines; ++y) {
        uint8* f = (uint8*) from->pixels + y*from->pitch;
        uint8* t = MT::add((uint8*) to->pixels + (2*y+target.y)*to->pitch,
                           target.x);
        for (int x = 0; x < cols; ++x) {
            typename MT::pix_type v = MT::peek(f); f = MT::add(f, 1);
            MT::poke(t, v);
            MT::poke(MT::add(t, 1), v);
            MT::poke(t + to->pitch, v);
            MT::poke(MT::add(t, 1) + to->pitch, v);
            t = MT::add(t, 2);
        }
    }
}

template<class MT>
void
blit4(SDL_Surface* to, SDL_Surface* from, GfxRect target)
{
    /* clip is in pixmap coordinates */
    target.clipSurface(to);
    
    int lines = target.h / 4;
    int cols = target.w / 4;
    if (lines > from->h)
        lines = from->h;
    if (cols > from->w)
        cols = from->w;

    for (int y = 0; y < lines; ++y) {
        for (int yy = 0; yy < 4; ++yy) {
            uint8* f = (uint8*) from->pixels + y*from->pitch;
            uint8* t = MT::add((uint8*) to->pixels + (yy+4*y+target.y)*to->pitch, target.x);
            for (int x = 0; x < cols; ++x) {
                typename MT::pix_type v = MT::peek(f); f = MT::add(f, 1);
                MT::poke(t, v); t = MT::add(t, 1);
                MT::poke(t, v); t = MT::add(t, 1);
                MT::poke(t, v); t = MT::add(t, 1);
                MT::poke(t, v); t = MT::add(t, 1);
            }
        }
    }
}

/*
  Local Variables:
  mode: c++
  End:
 */
