/**
  *  \file picedit/tools.cc
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <algorithm>
#include "picedit/tools.h"

int PETool::colors[2];
int PETool::swap;

PETool::~PETool()
{ }

void
PEPaint::click(Ptr<GfxCanvas> c, GfxPoint pt)
{
    prev = pt;
    c->drawPixel(pt.x, pt.y, getColor(), SDL_ALPHA_OPAQUE);
}

void
PEPaint::drag(Ptr<GfxCanvas> c, GfxPoint pt)
{
    GfxParameters parm;
    parm.color = getColor();
    drawLine(c, parm, prev.x, prev.y, pt.x, pt.y);
    prev = pt;
}

void
PEPaint::release(Ptr<GfxCanvas> c, GfxPoint pt)
{
    GfxParameters parm;
    parm.color = getColor();
    drawLine(c, parm, prev.x, prev.y, pt.x, pt.y);
}

void
PELine::click(Ptr<GfxCanvas> c, GfxPoint pt)
{
    prev = pt;
    c->drawPixel(pt.x, pt.y, getColor(), SDL_ALPHA_OPAQUE);
}

void
PELine::drag(Ptr<GfxCanvas> c, GfxPoint pt)
{
    GfxParameters parm;
    parm.color = getColor();
    drawLine(c, parm, prev.x, prev.y, pt.x, pt.y);
}

void
PELine::release(Ptr<GfxCanvas> c, GfxPoint pt)
{
    drag(c, pt);
}

void
PERect::click(Ptr<GfxCanvas> c, GfxPoint pt)
{
    prev = pt;
    c->drawPixel(pt.x, pt.y, getColor(), SDL_ALPHA_OPAQUE);
}

void
PERect::drag(Ptr<GfxCanvas> c, GfxPoint pt)
{
    GfxRect r;
    r.x = std::min(pt.x, prev.x);
    r.y = std::min(pt.y, prev.y);
    r.w = std::max(pt.x, prev.x) - r.x + 1;
    r.h = std::max(pt.y, prev.y) - r.y + 1;

    GfxParameters parm;
    parm.color = getColor();
    drawRectangle(c, parm, r);
}

void
PERect::release(Ptr<GfxCanvas> c, GfxPoint pt)
{
    drag(c, pt);
}

void
PEBar::click(Ptr<GfxCanvas> c, GfxPoint pt)
{
    prev = pt;
    c->drawPixel(pt.x, pt.y, getColor(), SDL_ALPHA_OPAQUE);
}

void
PEBar::drag(Ptr<GfxCanvas> c, GfxPoint pt)
{
    GfxRect r;
    r.x = std::min(pt.x, prev.x);
    r.y = std::min(pt.y, prev.y);
    r.w = std::max(pt.x, prev.x) - r.x + 1;
    r.h = std::max(pt.y, prev.y) - r.y + 1;

    drawSolidBar(c, r, getColor());
}

void
PEBar::release(Ptr<GfxCanvas> c, GfxPoint pt)
{
    drag(c, pt);
}
