/**
  *  \file picedit/tools.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_PICEDIT_TOOLS_H
#define PCC_V2_PICEDIT_TOOLS_H

#include "gfx/gfx.h"
#include "cpluslib/string.h"
#include "gfx/palette.h"

class PETool {
    static int swap;
    static int colors[2];
    bool needs_preview;
    string_t name;
 public:
    PETool(bool needs_preview, string_t name)
        : needs_preview(needs_preview), name(name) { }
    virtual ~PETool();
    virtual void click(Ptr<GfxCanvas> c, GfxPoint pt) = 0;
    virtual void drag(Ptr<GfxCanvas> c, GfxPoint pt) = 0;
    virtual void release(Ptr<GfxCanvas> c, GfxPoint pt) = 0;

    bool needsPreview() const { return needs_preview; }
    const string_t& getName() const { return name; }
    
//     static void setColor(uint32 c) { colors[0] = c; }
//     static void setBgColor(uint32 c) { colors[1] = c; }
    static int getColor() { return standard_colors[colors[swap]]; }
    static int getBgColor() { return standard_colors[colors[1-swap]]; }
    static void setSwapColors(bool b) { swap = b; }

    /* accessors for color selector */
    static void setColorN(int index, int c) { colors[index] = c; }
    static int getColorN(int index) { return colors[index]; }
};

class PEPaint : public PETool {
    GfxPoint prev;
 public:
    PEPaint() : PETool(false, "Paint") { }
    void click(Ptr<GfxCanvas> c, GfxPoint pt);
    void drag(Ptr<GfxCanvas> c, GfxPoint pt);
    void release(Ptr<GfxCanvas> c, GfxPoint pt);
};

class PELine : public PETool {
    GfxPoint prev;
 public:
    PELine() : PETool(true, "Line") { }
    void click(Ptr<GfxCanvas> c, GfxPoint pt);
    void drag(Ptr<GfxCanvas> c, GfxPoint pt);
    void release(Ptr<GfxCanvas> c, GfxPoint pt);
};

class PERect : public PETool {
    GfxPoint prev;
 public:
    PERect() : PETool(true, "Rectangle") { }
    void click(Ptr<GfxCanvas> c, GfxPoint pt);
    void drag(Ptr<GfxCanvas> c, GfxPoint pt);
    void release(Ptr<GfxCanvas> c, GfxPoint pt);
};

class PEBar : public PETool {
    GfxPoint prev;
 public:
    PEBar() : PETool(true, "Solid Bar") { }
    void click(Ptr<GfxCanvas> c, GfxPoint pt);
    void drag(Ptr<GfxCanvas> c, GfxPoint pt);
    void release(Ptr<GfxCanvas> c, GfxPoint pt);
};

#endif
