/**
  *  \file resmgr/res-cc1.cc
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <cstdlib>
#include "resmgr/res-cc1.h"
#include "util/nls.h"
#include "util/except.h"
#include "cpluslib/format.h"

const TStructureParser ResourceStream::Entry::parser[] = {
    { T_UINT16, 1, offsetof(ResourceStream::Entry, id),  0, 0 },
    { T_UINT32, 1, offsetof(ResourceStream::Entry, pos), 0, 0 },
    { T_UINT32, 1, offsetof(ResourceStream::Entry, len), 0, 0 },
    { T_END, 0, 0, 0, 0 }
};

const TStructureParser ResourceStream::Header::parser[] = {
    { T_UINT16, 1, offsetof(ResourceStream::Header, magic),    0, 0 },
    { T_UINT32, 1, offsetof(ResourceStream::Header, dirpos),   0, 0 },
    { T_UINT16, 1, offsetof(ResourceStream::Header, nentries), 0, 0 },
    { T_END, 0, 0, 0, 0 }
};

ResourceStream::ResourceStream(Ptr<Stream> aworker)
    : SubStreamBase(aworker)
{
    aworker->seek(0);
    if (!getStructure(aworker, header)
        || header.magic != 0x5A52 /* 'RZ' */
        || !aworker->seek(header.dirpos))
    {
        throw FileFormatException(format(_("File `%s' is not a resource file"),
                                         aworker->getName()));
    }
    if (header.nentries) {
        uint32      rdsize = Entry::size * uint32(header.nentries);
        Array<char> tmp(rdsize);
        if (aworker->read(tmp.ptr(), rdsize) != rdsize)
            throw FileTooShortException(aworker->getName());
        
        rd.ensureSize(header.nentries);
        const char* p = tmp.ptr();
        for (unsigned i = 0; i < header.nentries; ++i)
            p = getStructure(p, rd[i]);
    }
    setDisallowedOperations(C_WRITE);
}

ResourceStream::~ResourceStream()
{ }

bool
ResourceStream::seekId(uint16 id)
{
    for (unsigned i = 0; i < header.nentries; ++i) {
        if (id == rd[i].id) {
            setRange(rd[i].pos, rd[i].len);
            seek(0);
            return true;
        }
    }
    return false;
}


ResProviderResFile::ResProviderResFile(Ptr<Stream> s)
    : res(new ResourceStream(s))
{ }

ResProviderResFile::ResProviderResFile(string_t name)
    : res(new ResourceStream(Stream::openFile(name, Stream::C_READ)))
{ }

ResProviderResFile::~ResProviderResFile()
{ }

Ptr<Stream>
ResProviderResFile::getEntry(ResID id, ResClass rc)
{
    static const int bases[] = {
        0,                      // rt_Misc,
        0,                      // rt_Ship,
        200,                    // rt_Person,
        3000,                   // rt_Climate,
        4000,                   // rt_Fighter,
        4600,                   // rt_Torp,
        4700,                   // rt_TorpLauncher
        4800,                   // rt_Beam
    };

    /* FIXME: right now, we only have normal pictures */
    if (rc != rc_Normal)
        return 0;
    
    int pcc_id = bases[getResType(id)] + getResIndex(id);
    if (res->seekId(pcc_id))
        return res;
    else
        return 0;
}
