/**
  *  \file resmgr/res-wp.cc
  *  \brief Winplan Resource Provider
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "resmgr/res-wp.h"
#include "cpluslib/format.h"

/*! \class ResProviderWinplanBMP
    \brief Winplan Resource Provider

    This resource provider provides access to a Winplan "bmp" directory.
    It reads the ship pictures. */

ResProviderWinplanBMP::ResProviderWinplanBMP(Ptr<Directory> dir)
    : dir(dir)
{ }

ResProviderWinplanBMP::ResProviderWinplanBMP(string_t s)
    : dir(Directory::openDir(s))
{ }

ResProviderWinplanBMP::~ResProviderWinplanBMP()
{ }

Ptr<Stream>
ResProviderWinplanBMP::getEntry(ResID id, ResClass c)
{
    /* we only know one type this time: ships */
    if (getResType(id) == rt_Ship && c == rc_Normal)
        return dir->openFileNT(format("vpl%d.bmp", getResIndex(id)-1000),
                               Stream::C_READ);
    else
        return 0;
}
