/**
  *  \file resmgr/res-wpvcr.cc
  *  \brief Winplan WPVCR.DLL Resource Provider
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "resmgr/res-wpvcr.h"
#include "util/nls.h"
#include "util/except.h"
#include "fileio/io.h"
#include "cpluslib/assert.h"
#include "cpluslib/format.h"

/** \class ResProviderWPVCR
    \brief Winplan WPVCR.DLL Resource Provider

    This class provides functionality for reading the WPVCR.DLL file
    from Winplan. That file contains pictures to use in the VCR
    (left/right moving small ships). */

ResProviderWPVCR::ResProviderWPVCR(Ptr<Stream> file)
    : file(new SubStream(file))
{
    init();
}

ResProviderWPVCR::ResProviderWPVCR(string_t name)
    : file(new SubStream(Stream::openFile(name, Stream::C_READ)))
{
    init();
}

ResProviderWPVCR::~ResProviderWPVCR()
{ }

/** Initializer. Load the directories and configure the SubStream
    object. */
void
ResProviderWPVCR::init()
{
    ASSERT(file);
    char buffer[ENTRIES * 4];
    file->setDisallowedOperations(Stream::C_WRITE);
    file->setRange(0, -1);

    for (int i = 0; i < 2; ++i) {
        file->seek(1000 * i);
        if (file->read(buffer, sizeof buffer) != sizeof buffer)
            throw FileFormatException(format(_("File `%s' is invalid"), file->getName()));
        getUint32(buffer, offsets[i], ENTRIES);
        
        file->seek(1000 * i + 649);
        if (file->read(buffer, sizeof buffer / 2) != sizeof buffer / 2)
            throw FileFormatException(format(_("File `%s' is invalid"), file->getName()));
        getUint16(buffer, sizes[i], ENTRIES);

        for (int j = 0; j < ENTRIES; ++j)
            if (offsets[i][j] && sizes[i][j])
                --offsets[i][j];
    }
}

Ptr<Stream>
ResProviderWPVCR::getEntry(ResID id, ResClass rc)
{
    int side;

    /* we have 160 ship images only */
    if (getResType(id) != rt_Ship || getResIndex(id) < 1001 || getResIndex(id) > 1000+ENTRIES)
        return 0;

    /* we only know the left/right moving ones */
    if (rc == rc_LeftSmall)
        side = 1;
    else if (rc == rc_RightSmall)
        side = 0;
    else
        return 0;

    /* do we have that one? */
    int index = getResIndex(id) - 1001;
    if (sizes[side][index] && offsets[side][index]) {
        file->setRange(offsets[side][index], sizes[side][index]);
        return file;
    } else {
        return 0;
    }
}
