/**
  *  \file resmgr/resgfx.cc
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <map>
#include "resmgr/resmgr.h"

typedef std::map<std::pair<ResID, ResClass>, Ptr<GfxPixmap> > res_cache_t;
static res_cache_t res_cache;

typedef std::map<string_t, Ptr<GfxPixmap> > res_name_cache_t;
static res_name_cache_t res_name_cache;

Ptr<GfxPixmap>
ResProvider::getPixmapNS(ResID id, ResClass rc)
{
    res_cache_t::const_iterator i = res_cache.find(std::make_pair(id, rc));
    if (i != res_cache.end())
        return i->second;

    Ptr<GfxPixmap> pix = loadImage(getEntry(id, rc));
    if (pix)
        res_cache.insert(std::make_pair(std::make_pair(id, rc), pix));
    return pix;
}

Ptr<GfxPixmap>
ResProvider::getPixmap(ResID id, ResClass rc)
{
    Ptr<GfxPixmap> pix = getPixmapNS(id, rc);
    // FIXME: if (!pix) synthesize...
    return pix;
}

Ptr<GfxPixmap>
ResProvider::getNamedPixmap(string_t name)
{
    res_name_cache_t::const_iterator i = res_name_cache.find(name);
    if (i != res_name_cache.end())
        return i->second;

    // FIXME: maybe we should convert to screen format here
    Ptr<GfxPixmap> pix = loadImage(getNamedEntry(name));
    res_name_cache.insert(std::make_pair(name, pix));
    return pix;
}

Ptr<GfxPixmap>
ResProvider::getNamedPixmap(string_t name, int index)
{
    Ptr<GfxPixmap> pix = getNamedPixmap(name + itoa(index));
    if (!pix)
        return getNamedPixmap(name);
    return pix;
}
