/**
  *  \file resmgr/resmgr.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_RESMGR_RESMGR_H
#define PCC_V2_RESMGR_RESMGR_H

#include <iosfwd>
#include <map>
#include "cpluslib/misc.h"
#include "cpluslib/smartptr.h"
#include "cpluslib/types.h"
#include "fileio/dirs.h"
#include "fileio/streams.h"
#include "gfx/pixmap.h"
#include "cpluslib/string.h"

enum ResType {
    rt_Misc,                    /* these map 1:1 to PCC 1.x cc.res entries */
    rt_Ship,                    /* index is Hull().Image [i.e. +1000!]*/
    rt_Person,                  /* index is race */
    rt_Climate,                 /* index is 100-temp */
    rt_Fighter,                 /* index is race (1..11) */
    rt_Torp,                    /* index is type (1..10) */
    rt_TorpLauncher,            /* index is type */
    rt_Beam                     /* index is type */
};

enum ResClass {
    rc_Normal,                  /* normal image */
    rc_LeftSmall,               /* left-moving small image */
    rc_RightSmall               /* right-moving small image */
    /* wireframe, animation, ... */
};

typedef uint32 ResID;

inline ResID   mkResId(ResType rt, int index) { return 65536U*rt + index; }
inline ResType getResType(ResID id) { return ResType(id >> 16); }
inline int     getResIndex(ResID id) { return id & 65535U; }

class ResManager;

class ResProvider : public Object {
    friend class ResManager;
    Ptr<ResProvider> next;
    Ptr<GfxPixmap> getPixmapNS(ResID id, ResClass rc);
 public:
    ResProvider();
    ~ResProvider();
    virtual Ptr<Stream> getEntry(ResID id, ResClass rc);
    virtual Ptr<Stream> getNamedEntry(string_t name);

    Ptr<GfxPixmap> getPixmap(ResID id, ResClass rc);
    Ptr<GfxPixmap> getNamedPixmap(string_t name);
    Ptr<GfxPixmap> getNamedPixmap(string_t name, int id);
};

class ResDirectory : public ResProvider {
    typedef std::map<string_t, string_t> AliasMap;
    static AliasMap aliases;
    Ptr<Directory> dir;

    void loadAliases(std::istream&);
 public:
    ResDirectory(Ptr<Directory> dir);
    ~ResDirectory();

    Ptr<Stream> getNamedEntry(string_t name);
};

class ResManager : public ResProvider {
    Ptr<ResProvider> first;
    static ResManager instance;
 public:
    ResManager();
    ~ResManager();
    void addProvider(Ptr<ResProvider> prov);
    Ptr<Stream> getEntry(ResID id, ResClass rc);
    Ptr<Stream> getNamedEntry(string_t name);

    static ResManager& getInstance() { return instance; }
};

/* resmgrcf.cc */
bool loadResourceConfig(ResManager&, std::istream&);

inline Ptr<Stream> getResourceEntry(ResID id, ResClass rc) { return ResManager::getInstance().getEntry(id, rc); }
inline Ptr<GfxPixmap> getResourcePixmap(ResID id, ResClass rc) { return ResManager::getInstance().getPixmap(id, rc); }
inline Ptr<Stream> getNamedResourceEntry(string_t name) { return ResManager::getInstance().getNamedEntry(name); }

#endif
