#!/usr/bin/perl -w
#
#  Convert the PHost/PDK config definitions for use with PCC II
#

$CONFIG_S_H = "game/config-s.h";
$CONFIG_S_INC = "game/config-s.inc";

%typexl = (Uns16 => int16,
	   Int16 => int16,
	   Uns32 => int32,
	   Int32 => int32,
	   Boolean => "bool",
           Tristate => "TTristate",
           # for a clients, TSX is bool and External is Yes. We accept
           # External for all boolean options.
           TristateX => "bool",
	   Language_Def => string_t,
	   BuildQueue_Def => string_t,
	   ScoreMethod_Def => string_t);
%typeid = (int16 => T_INT16,
	   int32 => T_INT32,
	   "bool" => T_BOOL,
	   "string_t" => T_STRING,
           "TTristate" => T_TRISTATE);

$struct = "";
$conv = "";
$enum = "";
%thost = ();

# Fetch THost configuration
open TCONF, "<phost/tconfig.hi" or die "can't open tconfig.hi";
while (<TCONF>) {
    if (/^TCDefine\((\S+),\s*(\S+),\s*(\S+),\s*(\S+)\)/) {
	my ($typ, $count, $ofs, $name) = ($1, $2, $3, $4);
	my $typxl = $typexl{$typ};
	die "unknown type `$typ'" if !defined $typxl;
	$thost{$name} = "$typeid{$typxl}, $count, $ofs";
    } elsif(/^TCDefine/) {
	die "not matched: $_";
    }
}
close TCONF;

# PHost configuration
if (!@ARGV) {
    if (-r "phost/config.hi4") {
        push @ARGV, "phost/config.hi4";
    } else {
        push @ARGV, "phost/config.hi";
    }
    if (-r "phost/addconfig.hi") {
        push @ARGV, "phost/addconfig.hi";
    }
}
foreach (@ARGV) {
    open CONF, "< $ARGV[0]" or die "can't open $ARGV[0]: $!";
    while(<CONF>) {
        process_line ();
    }
    close CONF;
}

foreach (sort keys %conv) {
    $conv .= "    { \"$_\", $conv{$_}";
    $enum .= "    PCI_$orig{$_},\n";
}
foreach (sort keys %alias) {
    $alias .= "   { \"$_\", PCI_$alias{$_} },\n";
}

open CH, "> $CONFIG_S_H " or die "can't create $CONFIG_S_H";
print CH <<EOF
struct TGameConfig {
$struct
};

enum TConfigIndex {
$enum
    PCI_MAX
};
EOF
;
close CH;

open CC, "> $CONFIG_S_INC" or die "can't create $CONFIG_S_INC";
print CC "/* generated automatically, do not edit */

static const TPConfigConvert config_info[] = {
$conv
};

static const TPConfigAliasTable config_alias[] = {
$alias
};
";
close CC;

sub do_define {
    my ($key, $count, $type, $default, $array, $min, $max) = @_;
    $count = repl_const($count);
    $min = repl_const($min);
    $max = repl_const($max);
    if ($type eq "char" || $type eq "String") {
        $typexl = "string_t";
        $typesuf = "";
        $count = 1;
        $min = $max = 0;
    } else {
        $typexl = $typexl{$type};
        die "unknown type `$type'" if not defined $typexl;
        if ($count ne 1) {
            $typesuf = "[$count]";
        } else {
            $typesuf = "";
        }
    }
    if ($typexl eq "string_t") {
        $min = $max = 0;
    }
    $struct .= "    $typexl $key$typesuf;\n";
    $thost = $thost{$key} || "T_END, 0, 0";
    $conv{lc $key} = "$typeid{$typexl}, offsetof(TGameConfig, $key), $count, $array, $min, $max, \"$default\", $thost },\n";
    $orig{lc $key} = $key;
}

sub do_alias {
    my ($alias, $realname) = @_;
    $orig{lc $alias} = $alias;
    $alias{lc $alias} = $realname;
}

sub process_line {
    if (/^CFDefine\((\S+),\s*\S+,\s*(\S+),\s*(\S+),\s*"(.*)",\s*(\S+),\s*(\S+),\s*(\S+)\s*\)/) {
	my ($key, $count, $type, $default, $array, $min, $max)
	    = ($1, $2, $3, $4, lc $5, $6, $7);
        do_define ($key, $count, $type, $default, $array, $min, $max);
    } elsif(/^CFDefine4\((\S+),\s*\S+,\s*(\S+),\s*(\S+),\s*(\S+),\s*(\S+),\s*([^,]*),\s*([^,]*?),\s*"(.*)"\)/) {
        my ($key, $count, $type, $min, $max, $flags, $cond, $default)
	    = ($1, $2, $3, $4, $5, $6, $7, $8);
        my $array = ($flags =~ /ARRAYIZED/i) ? "true" : "false";
        do_define ($key, $count, $type, $default, $array, $min, $max);
    } elsif(/^CFAlias\((\S+),\s*(\S+)\)/) {
        do_alias ($1, $2);
    } elsif(/^CFDefine/) {
	print STDERR "unmatched CFDefine: $_";
    }
}

sub repl_const {
    my $x = shift;
    $x eq 'PLANET_NR' and $x = 'NUM_PLANETS';
    $x eq 'SHIP_NR' and $x = 'NUM_SHIPS';
    $x;
}

#print $struct;
#print $conv;
