#!/usr/bin/perl -w

use strict;
use vars qw(%defined %used);

foreach my $file (@ARGV) {
    open FILE, "< $file" or do {
	print STDERR "can't open $file: $!\n";
	next;
    };
    while (<FILE>) {
	if (/^\s*\#\s*(define|undef)\s+(\w+)/) {
	    $defined{$2} ||= $file;
	} elsif (/^\s*\#\s*include\s*\</) {
	    # ignore `#include <foo>'
	} elsif (/^\s*\#\s*(if|ifdef|ifndef|include|error|line|endif|else|elif|pragma)\b\s*(.*)/) {
	    my $text = $2;
	    $text =~ s/"([^\\]|\\.)*"/ /;
	    $text =~ s/'([^\\]|\\.)*'/ /;
	    foreach (split /\W+/, $text) {
		next if $_ eq "defined" || $_ eq "" || $_ =~ /^\d+$/;
		$used{$_} ||= $file;
	    }
	} elsif (/^\s*\#/) {
	    chomp;
	    print STDERR "$file: unknown pp command `$_'\n";
	}
    }
    close FILE;
}

foreach (sort keys %used) {
    next if defined $defined{$_};
    print "$_: used in $used{$_} but never defined\n";
}
