/**
  *  \file sound/sound.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_SOUND_SOUND_H
#define PCC_V2_SOUND_SOUND_H

#include <SDL_audio.h>
#include "cpluslib/misc.h"
#include "fileio/streams.h"
#include "cpluslib/types.h"

class SfxSampleData : public Object {
    uint8* sound_data;
    uint32 sound_size;

    void loadWav(Stream* file);
 public:
    SfxSampleData(Ptr<Stream> file);
    ~SfxSampleData();

    uint32 mix(uint8* dest_buf, int size, uint8 volume, uint8 pan, uint32 pos);
};

class SfxSample {
    SfxSampleData* data;
    int generation;
    static int current_generation;
 public:
    SfxSample(SfxSampleData* data = 0) : data(data), generation(current_generation) { }

    void play(uint8 volume, uint8 pan, int importance);
    bool isValid() const { return !isOutdated() && data != 0; }
    bool isOutdated() const { return current_generation != generation; }

    static void newGeneration() { ++current_generation; }
};

enum {
    pan_Left   = 0,
    pan_Center = 128,
    pan_Right  = 255,

    volume_Max = 255
};

struct SfxConfig {
    /** Sample rate in Hertz. */
    int  frequency;
    /** True iff stereo. */
    bool stereo;
    /** True iff 16-bit output wanted. */
    bool sixteen_bits;
    /** True iff sound enabled. */
    bool enabled;
    /** Headphone mode. */
    bool headphone;
    /** Reverse stereo. */
    bool reverse_stereo;
};

void initSound();
void doneSound();
void updateSoundStatus();
void configureSound(const SfxConfig& config);
const SfxConfig& getSoundConfig();

void loadSoundConfig();
void saveSoundConfig();

SfxSample requestSound(const char* name);
SfxSample requestSound(const char* name, int index);
SfxSample requestSound(string_t name);
SfxSample requestSound(string_t name, int index);

/* The following things are *not* part of the API */
extern SDL_AudioSpec sound_hardware;

#endif
