/*
 *  TEST: ConfigFile
 *
 *  This takes a config file name on the command line, and then offers
 *  an interactive interface to exercise the accessor functions.
 */
#include <iostream>
#include "util/confpars.h"

int main(int argc, char** argv)
{
    if (argc != 2) {
        std::cerr << "usage: testconfig FILE\n";
        return 1;
    }

    ConfigFile file;
    string_t line;
    
    file.load(argv[1]);
    std::cout << "?KEY = query value of KEY\n"
              << "=KEY = assign to KEY (value prompted on next line)\n"
              << "s    = save\n"
              << "q    = quit\n";
    while (getline(std::cin, line)) {
        if (!line.length())
            continue;
        if (line[0] == '?') {
            string_t out;
            if (file.getValueNC(line.substr(1), out))
                std::cout << "value is `" << out << "'\n";
            else
                std::cout << "not set\n";
        } else if (line[0] == '=') {
            string_t val;
            getline(std::cin, val);
            file.setValue(line.substr(1), val);
        } else if (line == "s") {
            file.save(argv[1]);
        } else if (line == "q") {
            break;
        } else {
            std::cout << "unrecognized command\n";
        }
    }
}
