/*
 *  TEST: expression evaluation
 *
 *  This takes a number of expressions on the command line.
 *  Each of them is evaluated; the result and byte-compilation result
 *  is displayed.
 */

#include <iostream>
#include "int/lex.h"
#include "int/data.h"
#include "int/expr.h"
#include "util/debug.h"
#include "int/bytecode.h"

class TestContext : public IntContext {
    static IntObject values[26];
 public:
    handle_t  lookUp(string_t name);
    IntObject read(handle_t);
    void      store(handle_t, IntObject);
    bool next() { return false; }
};

IntObject TestContext::values[26];

TestContext::handle_t
TestContext::lookUp(string_t name)
{
    if (name.length() == 1 && name[0] >= 'A' && name[0] <= 'Z')
        return name[0] - 'A';
    else
        return -1;
}

IntObject
TestContext::read(handle_t n)
{
    return values[n];
}

void
TestContext::store(handle_t n, IntObject v)
{
    values[n] = v;
}

int main(int argc, char** argv)
{
    initDebug();
    IntExecutionContext con;
    con.context = new TestContext();
    while (*++argv) {
        try {
            IntLexer lex(*argv);
            lex.lex();
            IntExprParser parser(lex);
            parser.parseExpr();
            IntExprNodePtr node = parser.pop();
            try {
                IntObject o = node->eval(con);
                std::cout << *argv << "\n-> ";
                switch(o.getType()) {
                 case res_Null:
                    std::cout << "Empty\n";
                    break;
                 case res_Int:
                    std::cout << "Int: " << o.getIntVal() << "\n";
                    break;
                 case res_Bool:
                    std::cout << "Bool: " << o.getIntVal() << "\n";
                    break;
                 case res_Float:
                    std::cout << "Float: " << o.getFloatVal() << "\n";
                    break;
                 case res_String:
                    std::cout << "String: " << o.getStringVal() << "\n";
                    break;
                 case res_Context:
                    std::cout << "Context\n";
                    break;
                 case res_Function:
                    std::cout << "Function\n";
                    break;
                 default:
                    std::cout << "UFO (type " << int(o.getType()) << ")\n";
                    break;
                }
            }
            catch(IntException& e) {
                std::cout << *argv << "\n-> Eval error: " << e.what() << "\n";
            }
            {
                std::cout << "=== Value Code ===\n";
                IntByteCodeObject bco;
                node->compileV(bco);
                bco.dump(std::cout);
            }
            {
                std::cout << "=== Sideeffect Code ===\n";
                IntByteCodeObject bco;
                node->compileS(bco);
                bco.dump(std::cout);
            }
        }
        catch(IntException& e) {
            std::cout << *argv << "\n-> Error: " << e.what() << "\n";
        }
    }
}
