/*
 *  TEST: IntLexer
 *
 *  This reads lines from standard input, splits them into tokens,
 *  and displays these on standard output.
 */

#include <iostream>
#include "int/lex.h"

int main()
{
    string_t s;
    while (getline(std::cin, s)) {
        IntLexer lex(s);
        std::cout << "[" << s << "]\n";
        try {
            IntTokenType t;
            while ((t = lex.lex()) != tt_Null) {
#define SHOW(what) case tt_##what: std::cout << "\t" #what << std::endl; break
                switch(t) {
                    SHOW(Null);    SHOW(Invalid); SHOW(NE);    SHOW(GE);
                    SHOW(LE);      SHOW(Asgn);    SHOW(AND);   SHOW(OR);
                    SHOW(XOR);     SHOW(NOT);     SHOW(MOD);   SHOW(Join);
                    SHOW(Hash);    SHOW(Plus);    SHOW(Minus); SHOW(Mul);
                    SHOW(RealDiv); SHOW(Div);     SHOW(Pot);   SHOW(Left);
                    SHOW(Right);   SHOW(Comma);   SHOW(EQ);    SHOW(LT);
                    SHOW(GT);      SHOW(Colon);   SHOW(Semi);  SHOW(Dot);
                 case tt_Bool:
                    std::cout << "\tBool: " << lex.getInt() << std::endl;
                    break;
                 case tt_Int:
                    std::cout << "\tInt: " << lex.getInt() << std::endl;
                    break;
                 case tt_Float:
                    std::cout << "\tFloat: " << lex.getFloat() << std::endl;
                    break;
                 case tt_String:
                    std::cout << "\tString: " << lex.getString() << std::endl;
                    break;
                 case tt_Id:
                    std::cout << "\tId: " << lex.getString() << std::endl;
                    break;
                }
            }
        }
        catch(const IntException& e) {
            std::cout << "--> exception: " << e.what() << "\n";
        }
    }
}
