/*
 *  TEST: GResultFile, GInbox
 *
 *  This takes a RST file name on the command line, and outputs the
 *  section addresses and all incoming messages to stdout.
 */

#include <iostream>
#include "game/rst.h"
#include "game/msg.h"
#include "fileio/streams.h"
#include "util/except.h"
#include "cpluslib/format.h"

int main(int argc, char** argv)
{
    if (argc != 2) {
        std::cerr << "usage: " << argv[0] << " <file.rst>\n";
        return 1;
    }

    try {
        Ptr<Stream> s = Stream::openFile(argv[1], Stream::C_READ);
        GResultFile rst(s);
        std::cout << "--- RST Version " << rst.getVersion() << " ---\n";
        for (int i = 0; i < 8; ++i)
            std::cout << format("Section %d is at %6d\n", i, rst.getSectionOffset(i));
        s->seek(rst.getSectionOffset(GResultFile::SEC_MESSAGES));
        GInbox inb;
        inb.loadInbox(s);
        std::cout << "--- There are " << inb.getCount() << " messages ---\n";
        for (int i = 1; i <= inb.getCount(); ++i) {
            std::cout << ">> Message #" << i << "\n"
                      << ">> Heading: " << inb.getHeading(i) << "\n"
                      << ">> Text:\n"
                      << inb.getText(i) << "\n";
        }
    }
    catch(std::exception& e) {
        std::cerr << "Bang! exception " << e.what() << "\n";
        return 1;
    }
}
