/*
 *  TEST: GfxMultiClip
 *
 *  This draws random rectangle, from back to front, exercising non-
 *  rectangular clipping and rectangle-set operations.
 */

#include <SDL/SDL.h>
#include <stdlib.h>
#include "gfx/gfx.h"
#include "gfx/multiclip.h"

int main(int, char**)
{
    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) != 0) {
        printf("error initializing SDL: %s\n", SDL_GetError());
        return 1;
    }
    atexit(SDL_Quit);
    
    SDL_Surface* sfc = SDL_SetVideoMode(640, 480, 32, SDL_HWPALETTE);
    if (!sfc) {
        printf("can't set video mode\n");
        return 1;
    }

    Ptr<GfxCanvas> s = new GfxSurface(sfc, false);
    Ptr<GfxMultiClip> m = new GfxMultiClip(s);
    m->add(GfxRect(sfc));
//#define m s

    while(1) {
        SDL_Event e;
        if (SDL_PollEvent(&e) && e.type == SDL_KEYDOWN)
            break;

        int x = rand() % 600, y = rand() % 400;
        int w = rand() % (640-x), h = rand() % (480-y);
        GfxRect r(x,y,w,h);
        m->lock();
        m->drawBar(r, SDL_MapRGB(sfc->format, rand() % 256, rand() % 256, rand() % 256), GFX_TRANSPARENT, SOLID_FILL, 255);
        m->remove(r);
        s->drawHLine(x, y, x+w, SDL_MapRGB(sfc->format, 255, 255, 255), ~0U, 128);
        s->drawVLine(x, y+1, y+h, SDL_MapRGB(sfc->format, 255, 255, 255), ~0U, 128);
        m->update(r);
        m->unlock();
//        SDL_Delay(55);
    }
}
