/*
 *  TEST: getPartialStructure
 *
 *  This takes a ship.dat on the command line and emits the first
 *  ship's Id, engine, ... to stdout. It then hacks the structure
 *  and outputs parts again. The hull type will have been changed.
 */

#include <iostream>
#include <cstdlib>
#include "fileio/io.h"
#include "fileio/streams.h"
#include "game/struct.h"

struct die {
    operator std::ostream&() const { return std::cerr; }
    //template<class T> std::ostream& operator
    ~die() { std::exit(1); }
};

const char hack[] = { 1, 2, 3, 4 };

int main(int argc, char** argv)
{
    if (argc != 2)
        die() << "usage: " << argv[0] << " shipX.dat\n";

    Ptr<Stream> f = Stream::openFile(argv[1], Stream::C_READ);
    TInt16 n;
    TShip  ship;

    if (!getStructure(f, n))
        die() << "can't read count\n";
    std::cout << n.value << " ships\n";
    if (!getStructure(f, ship))
        die() << "can't read ship\n";
    std::cout << "ship #" << ship.ship_id << " has " << ship.engine << "-type engines and hull type "
              << ship.hull << " and beam type " << ship.beam_type << "\n";
    const char* p = getPartialStructure(hack, &ship, TShip::parser, 18, sizeof hack);
    std::cout << "hacked ship #" << ship.ship_id << " has " << ship.engine << "-type engines and hull type "
              << ship.hull << " and beam type " << ship.beam_type << "\n";
    std::cout << "we have consumed " << (p-hack) << " bytes\n";

    ship.transfer.id = 123;
    std::cout << "ship.transfer.id set to " << ship.transfer.id << "\n";
    p = getPartialStructure(hack, &ship, TShip::parser, 101, 2);
    std::cout << "hacked ship transfer is " << ship.transfer.id << "\n";
    std::cout << "we have consumed " << (p-hack) << " bytes\n";
}
