/*
 *  TEST: sound mixer, SfxSample, etc.
 *
 *  This takes up to 10 .wav files on the command line. It will then read
 *  lines from stdin; each '0'..'9' plays the respective sample at
 *  "center" position, 50% volume.
 *
 *  This tests mixing, voice combination, sound loading. In particular,
 *  one can nicely observe CPU usage with this one.
 */

#include <iostream>
#include "sound/sound.h"
#include "util/except.h"
#include "util/console.h"
#include "cpluslib/string.h"

int main(int argc, char** argv)
{
    try {
        enableStdioConsole();
        initSound();

        int i = 0;
        SfxSample s[10];
        while (*++argv && i < 10)
            s[i++] = new SfxSampleData(Stream::openFile(*argv, Stream::C_READ));

        while ((i = getchar()) != -1 && i != 'q') {
            updateSoundStatus();
            if (i >= '0' && i <= '9') {
                i -= '0';
                cout << "playing " << i << "\n";
                s[i].play(128, pan_Center, 1);
            }
        }

        doneSound();
    }
    catch(std::exception& e) {
        std::cout << "bang! " << e.what() << "\n";
    }
}
