/**
  *  \file ui/commands.h
  *  \brief Command Numbers
  *
  *  This module defines the common cm_XXX constants.
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UI_COMMANDS_H
#define PCC_V2_UI_COMMANDS_H

#include "ui/event.h"

/** Standard Command Codes. Assigning command codes while not
    causing conflicts is a major headache. So I decided on the
    following scheme:
    - all positive values are available to every window
    - [-1000,0) contains common commands and those used by the kernel
    - (-inf,-1000) is assigned to windows that wish their own range

    Note that these are not actually command codes; they are primarily
    widget ids. Because they are called cm_XXX in PCC 1.x, I also call
    them so here.

    Some command codes remain, for events posted internally (cm_INT). */

class UIWidget;

enum {
    /* Standard Codes */
    cm_Enter     = -1000,       ///< "Enter", "Confirm"
    cm_Escape,                  ///< "ESC", "Cancel"
    cm_Yes,                     ///< "Yes"
    cm_No,                      ///< "No"
    cm_Close,                   ///< "Close", "Exit"
    cm_Help,                    ///< "Help"
    cm_Next,                    ///< scrolling...
    cm_Prev,
    cm_NextMarked,
    cm_PrevMarked,
    cm_Print,                   ///< "Print"
    cm_Inc,                     ///< "Increment", "+"
    cm_Dec,                     ///< "Decrement", "-"
    cm_Tab,                     ///< "Table"

    /* Internal commands */
    cm_INT_QueryReferencePoint,

    /* Aliases */
    cm_Cancel = cm_Escape,
    cm_Exit   = cm_Close,

    /* Commands available for current window */
    cm_ControlScreenFirst   = -2000, // 1000
    cm_VcrSelectionFirst    = -2100  // 100
};

struct TReferencePointArg {
    UIEvent    event;
    int        x, y;
    UIWidget*  widget;

    TReferencePointArg(const UIEvent& e)
        : event(e), x(0), y(0), widget(0) { }
};

#endif
