/**
  *  \file ui/layout.h
  *  \brief Layout Management
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UI_LAYOUT_H
#define PCC_V2_UI_LAYOUT_H

#include <vector>
#include "ui/ui.h"

/** Basic Layout Manager Interface. Encapsulates the functionality
    of arranging a widget container's children. */
class UILayoutManager : public Object {
 public:
    /** Arrange a container's children. This does not resize
        the container itself. */
    virtual void doLayout(UIWidgetContainer* container, GfxRect size) = 0;
    /** Get layout information. Compute preferred and minimum sizes,
        just like UIWidget::getLayoutInfo(). */
    virtual void getLayoutInfo(UIWidgetContainer* container, UIWidget::LayoutInfo& info) = 0;
};

// horizontal box
class UIHBoxLayout : public UILayoutManager {
    void doLayoutWithWidths(UIWidgetContainer* container, GfxRect size,
                            std::vector<int>& sizes, int space, int outer);
    int space, outer;
 public:
    UIHBoxLayout(int aspace = 0, int aouter = 0);
    void doLayout(UIWidgetContainer* container, GfxRect size);
    void getLayoutInfo(UIWidgetContainer* container, UIWidget::LayoutInfo& info);
};

// vertical box
class UIVBoxLayout : public UILayoutManager {
    void doLayoutWithHeights(UIWidgetContainer* container, GfxRect size,
                             std::vector<int>& sizes, int space, int outer);
    int space, outer;
 public:
    UIVBoxLayout(int aspace = 0, int aouter = 0);
    void doLayout(UIWidgetContainer* container, GfxRect size);
    void getLayoutInfo(UIWidgetContainer* container, UIWidget::LayoutInfo& info);
};

#endif
