/**
  *  \file ui/listbox.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UI_LISTBOX_H
#define PCC_V2_UI_LISTBOX_H

#include <list>
#include <utility>
#include "ui/ui.h"
#include "cpluslib/sigslot.h"

class UIScrollable : public UIWidget {
    int pos, top, size, page;
 protected:
    void setCurrent(int i)  { pos  = i; }
    void setTop(int i)      { top  = i; }
    void setSize(int i)     { size = i; }
    void setPageSize(int i) { page = i; }
    void adjustPosition();
 public:
    UIScrollable(int id);
    ~UIScrollable();
    virtual void scrollTo(int pos, int corr) = 0;
    virtual bool isAccessible(int pos);
    int getCurrent() const  { return pos; }
    int getTop() const      { return top; }
    int getSize() const     { return size; }
    int getPageSize() const { return page; }

    Signal1<int> sig_moved;
};

class UIListbox : public UIScrollable {
    int lines, width;           /* user preferences */
    int item_height;
    bool mdown;
 protected:
    void setItemHeight(int h) { item_height = h; }
 public:
    UIListbox(int id, int lines, int width, int items);

    void scrollTo(int pos, int corr);
    void draw(Ptr<GfxCanvas> can, UpdateInfo& ui);
    void stateChanged(int state, bool enable);
    bool handleEvent(const UIEvent& e, bool second_pass);
    void updateCurrent();
    void setExtent(GfxRect newpos, bool by_layout);
    void setDefaultHeight();

    void getLayoutInfo(LayoutInfo& info);

    int getItemHeight() const { return item_height; }

    virtual void drawPart(Ptr<GfxCanvas> can, int from, int to) = 0;

    Signal1<int> sig_dblclick, sig_menu;

    enum {
        lb_Menu         = 2*st_Last,
        lb_NoPageKeys   = 4*st_Last,
        lb_KeyboardMenu = 8*st_Last
    };

    bool       doStandardListbox(string_t title, int help_id=0);
    bool       doStandardMenu(int x, int y);
    bool       doStandardMenu(const UIEvent& ref_event);
};

class UIStringList : public Object {
    typedef std::list<std::pair<int,string_t> > list_t;
    list_t data;
 public:
    typedef list_t::const_iterator iterator;

    UIStringList();
    ~UIStringList();
    void       add(int key, const string_t& s);
    void       add(int key, const char* s);

    iterator   getItem(int index) const;
    iterator   getItemByKey(int key) const;
    int        getItemIndexByKey(int key) const;

    iterator   begin() const    { return data.begin(); }
    iterator   end() const      { return data.end(); }
    int        getCount() const { return data.size(); }
};

class UIStandardListbox : public UIListbox {
    UIStringList& data;
 public:
    UIStandardListbox(int id, int lines, int width, UIStringList& data);
    ~UIStandardListbox();

    void       drawPart(Ptr<GfxCanvas> can, int from, int to);
    int        getKey() const { return data.getItem(getCurrent())->first; }
    void       scrollToKey(int key);
};

class UIScrollbar : public UIWidget {
    UIScrollable&     scrl;
    void*             cookie;
    int               bf[2];

    void sync();
    void scrolled(int);
    void setBF(int id, int flags);
 public:
    UIScrollbar(UIScrollable& scrl);
    ~UIScrollbar();

    void draw(Ptr<GfxCanvas> can, UpdateInfo& ui);
    void getLayoutInfo(LayoutInfo& info);
    bool handleEvent(const UIEvent& e, bool second_pass);
    void stateChanged(int flags, bool enable);
};

#endif
