/**
  *  \file ui/msgbox.cc
  *  \brief Implementation of Standard Message Boxes
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "ui/msgbox.h"
#include "gfx/font.h"
#include "ui/util.h"
#include "ui/window.h"
#include "gfx/palette.h"
#include "ui/uiglobal.h"
#include "ui/widgets.h"

namespace {
/** Multi-Line Static Text. */
// FIXME: This class should be moved elsewhere.
class UIMultilineStatic : public UIFixedWidget {
    int font;
    string_t text;
 public:
    UIMultilineStatic(int afont, string_t atext, int aid = 0);
    void draw(Ptr<GfxCanvas> can, UpdateInfo& ui);
};
}

UIMultilineStatic::UIMultilineStatic(int afont, string_t atext, int aid)
    : UIFixedWidget(aid), font(afont), text(atext)
{ }

void
UIMultilineStatic::draw(Ptr<GfxCanvas> can, UpdateInfo& ui)
{
    GfxParameters parm;
    parm.color = standard_colors[COLOR_BLACK];
    outTextFormatted(can, parm, fonts[0], extent.x, extent.y, extent.w, text);
    ui.update(getExtent());
}

/** Standard Message Box. A message box containing the specified text
    and title, and an "OK" button. The message box is modal. The text
    can contain '\n' to start new lines; it is automatically
    word-wrapped. */
void
messageBox(string_t text, string_t title)
{
    int width = fonts[0]->getTextWidth(text);
    int lines;
    
    if (width < 440) {
        lines = 1;
        if (width < 200)
            width = 200;
    } else {
        width = 400;
        lines = countLines(fonts[0], width, text);
    }

    int total_width = width + 40;
    int total_height = lines * fonts[0]->getTextHeight("Tp") + 80;

    UIWindow win(GfxRect(0, 0, total_width, total_height),
                 title, UIWindow::BLUE_WINDOW, COLOR_GRAY);
    UIMultilineStatic mls(0, text);
    UIButton but(0, "OK", SDLK_RETURN, UIButton::aStop);
    
    win.setState(UIWidget::st_Modal, true);
    win.add(GfxRect(20, 30, width, total_height-80), mls);
    win.add(GfxRect(total_width/2-40, total_height-35, 80, 25), but);

    ui_root->centerWidget(win);
    ui_root->add(win);
    ui_root->run();
    ui_root->close(win);
}
