/**
  *  \file ui/peerlink.cc
  *  \brief Peer links.
  *
  *  This module provides two classes, UIPeer and UIPeerLink<T>, who
  *  can be used to manage "weak" peer relationships of objects. When
  *  A uses B, B must be derived from UIPeer, and A contains an object
  *  of type UIPeerLink<TypeOfB>. When B dies before A, A's peer link
  *  automatically goes to null. I'm unsure whether this class is
  *  useful; it probably belongs in cpluslib/ not ui/.
  */

#include "ui/peerlink.h"

UIPeer::UIPeer()
{ }

UIPeer::~UIPeer()
{
    sig_death.raise();
}

UIBasicPeerLink::UIBasicPeerLink(UIPeer* p)
    : peer(p)
{
    if (peer)
        cookie = p->sig_death.add(this, &UIBasicPeerLink::onPeerDeath);
}

UIBasicPeerLink::~UIBasicPeerLink()
{
    if (peer)
        peer->sig_death.remove(cookie);
}

void
UIBasicPeerLink::onPeerDeath()
{
    peer = 0;
}

void
UIBasicPeerLink::set(UIPeer* p)
{
    if (p == peer)
        return;
    if (peer)
        peer->sig_death.remove(cookie), peer = 0;
    if (p)
        cookie = p->sig_death.add(this, &UIBasicPeerLink::onPeerDeath), peer = p;
}
