/**
  *  \file ui/widget-num.cc
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "ui/widgets.h"

UINumberSelector::UINumberSelector(int id, long low, long high, int step)
    : UIWidget(id), current(low), low(low), high(high), step(step)
{ }

UINumberSelector::~UINumberSelector()
{ }

void
UINumberSelector::setValue(long val)
{
    if (val < low)
        val = low;
    if (val > high)
        val = high;
    if (val != current) {
        current = val;
        drawWidget(false);
        sig_changed.raise(getId());
    }
}

bool
UINumberSelector::handleEvent(const UIEvent& e, bool second_pass)
{
    if (e.type == ev_Key && hasState(st_Focused)) {
        switch (e.key.code) {
         case '+':
         case SDLK_RIGHT:
            /* FIXME: prefix */
            setValue(getValue() + step);
            return true;
         case '-':
         case SDLK_LEFT:
            /* FIXME: prefix */
            setValue(getValue() - step);
            return true;
         case ss_Ctrl + '+':
         case ss_Ctrl + SDLK_RIGHT:
            setValue(getValue() + 100);
            return true;
         case ss_Ctrl + '-':
         case ss_Ctrl + SDLK_LEFT:
            setValue(getValue() - 100);
            return true;
         case ss_Shift + '+':
         case ss_Shift + SDLK_RIGHT:
            setValue(getValue() + 1);
            return true;
         case ss_Shift + '-':
         case ss_Shift + SDLK_LEFT:
            setValue(getValue() - 1);
            return true;
         case ss_Alt + '+':
         case ss_Alt + SDLK_RIGHT:
            setValue(high);
            return true;
         case ss_Alt + '-':
         case ss_Alt + SDLK_LEFT:
            setValue(low);
            return true;
        }
    }
    return UIWidget::handleEvent(e, second_pass);
}
