/**
  *  \file util/cmdline.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UTIL_CMDLINE_H
#define PCC_V2_UTIL_CMDLINE_H

#include <vector>
#include <cstddef>
#include "util/except.h"

//! Command line parser.
class CommandLineParser {
    char** argv;
    static const char* prog;

    const char* arg;
    bool traditional;

    const char* getNext();
 public:
    CommandLineParser(bool atrad = false);
    virtual ~CommandLineParser();
    void parse(char** aargv);

    virtual void parseStarted();
    virtual void parseFinished();
    virtual void handleNonoption(const string_t&) = 0;
    virtual void handleOption(const string_t&, bool have_arg) = 0;

    const char* getArg();

    static const char* getProgramName() { return prog; }
};

class StandardCommandLineParser : public CommandLineParser {
 public:
    struct Option {
        const char* name;
        void        (*handler)(StandardCommandLineParser& parser, const Option& opt);
        void*       user_data;
        bool        takes_arg;
        const char* help;
    };
    StandardCommandLineParser(bool atrad = false);
    ~StandardCommandLineParser();
    void addOptionSet(const Option* opt, std::size_t count);

    void handleOption(const string_t& opt, bool have_arg);
    void printHelp(std::ostream& os);

    virtual void handleError(const string_t& msg);
 private:
    std::vector<Option> options;
};

class CommandLineException : public UserException {
 public:
    CommandLineException(const string_t& str)
        : UserException(str) { }
};

/* Some default handlers */
template <class T, T value>
void optSetValue(StandardCommandLineParser&, const StandardCommandLineParser::Option& opt)
{
    *static_cast<T*>(opt.user_data) = value;
}

template <void (*func)()>
void optCallFunction(StandardCommandLineParser&, const StandardCommandLineParser::Option&)
{
    func();
}

#endif
