/**
  *  \file confpars.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CONFPARS_H
#define PCC_V2_CONFPARS_H

#include <iosfwd>
#include <string>
#include "cpluslib/string.h"

class ConfigParser {
 public:
    virtual void assign(int line_id, string_t name, string_t value,
                        string_t line) = 0;

    virtual void unparsed(int line_id, string_t line);
    virtual void error(int line_id, string_t msg);

    virtual ~ConfigParser() { }

    void parse(std::istream& in, string_t section, bool in_section,
               int line_id = 0);
};

class ConfigFile {
    ConfigFile(const ConfigFile&);
    ConfigFile& operator=(const ConfigFile&);

 public:
    struct Data;                // the implementation is private to this module
 private:
    Data& data;
 public:
    ConfigFile();
    ~ConfigFile();
    bool load(string_t file_name);
    bool load(string_t file_name, string_t section, bool in_section = true);
    void save(string_t file_name);
    void save(string_t file_name, string_t section, bool in_section = true);

    bool getValueNC(string_t key, string_t& ret) const;

    /* Setters */
    void setValue(string_t key, string_t value);
    void setValue(string_t key, const char* value);
    void setValue(string_t key, int value);
    void setValue(string_t key, bool value);

    /* High-level reading interface */
    bool getValue(string_t key, string_t& value);
    bool getValue(string_t key, int& value);
    bool getValue(string_t key, bool& value);
};

extern ConfigFile config_file;

void loadConfig();
void saveConfig();

#endif
