/**
  *  \file console.h
  *  \brief Console Declarations
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CONSOLE_H
#define PCC_V2_CONSOLE_H

#include <string>
#include <iostream>
#include "cpluslib/sigslot.h"
#include "cpluslib/string.h"

enum MessageClass {
    LOG_DEBUG,
    LOG_INFO,
    LOG_WARN,
    LOG_ERROR
};

class Console : public std::ostream {
    enum { DEFAULT_SCROLLBACK_SIZE = 100 };
 public:
    struct Message {
        MessageClass mc;
        string_t text;
        int format;
        Message(MessageClass amc, string_t atext, int aformat)
            : mc(amc), text(atext), format(aformat) { }
        Message()
            : mc(LOG_DEBUG), text(), format(0) { }
    };
 private:
    class Streambuf;
    friend class Streambuf;
    Streambuf* buf;
    
    int scrollback_length;
    Message* scrollback_buffer;
    int buf_head, buf_fill;
    
    void writeLine(MessageClass mc, string_t text, int format = 0);
    inline Message& getLineInternal(int n) const;
 public:
    Console();
    virtual ~Console();
    void write(MessageClass mc, string_t text, int format = 0);
    
    int getScrollbackSize() const { return scrollback_length; }
    void setScrollbackSize(int n);
    int getScrollbackFill() const { return buf_fill; }
    const Message& getLine(int n) const;
    void clear();

    /** Main Display Hook. Called when the console contents changes.
        \param arg1 the console which calls it
        \param arg2 number of lines that were added anew (in terms of
                    getLine(), lines arg2-1 .. 0). If zero, the console
                    was cleared (or something else that needs complete
                    redraw). */
    Signal2<const Console&, int> console_changed;
};

extern Console console;

void enableStdioConsole();
void disableStdioConsole();

#endif
