/**
  *  \file except.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_EXCEPT_H
#define PCC_V2_EXCEPT_H

#include <exception>
#include <stdexcept>
#include "cpluslib/string.h"

/* Standard exceptions:
   exception
     bad_alloc
     bad_exception
     ios::failure
     bad_typeid
     bad_cast
     runtime_error
       range_error
       overflow_error
       underflow_error
     logic_error
       length_error
       domain_error
       out_of_range
       invalid_argument */

/** Error opening a file. Message should be a user-readable message
    (`file: permission denied', `could not open FOOFILE') */
class FileOpenException : public std::runtime_error {
 public:
    FileOpenException(const string_t& awhat);
    FileOpenException(const char* awhat);
    FileOpenException(const FileOpenException&);
    ~FileOpenException() throw();
};

/** Invalid file format. The message should indicate which file. */
class FileFormatException : public std::runtime_error {
 public:
    FileFormatException(const string_t& awhat);
    FileFormatException(const char* awhat);
    FileFormatException(const FileFormatException&);
    ~FileFormatException() throw();
};

/** File too short. Convenience class. */
class FileTooShortException : public FileFormatException {
 public:
    FileTooShortException(const string_t& fname);
    FileTooShortException(const char* awhat);
    FileTooShortException(const FileTooShortException&);
    ~FileTooShortException() throw();
};

/** File operation failed. The message should indicate which file. */
class FileIOException : public std::runtime_error {
 public:
    FileIOException(const string_t& awhat);
    FileIOException(const char* awhat);
    FileIOException(const FileIOException&);
    ~FileIOException() throw();
};

/** Graphics operation failed. Those are usually fatal. The message
    text usually is SDL_GetError(). */
class GraphicsIOException : public std::runtime_error {
 public:
    GraphicsIOException(const string_t& awhat);
    GraphicsIOException(const char* awhat);
    GraphicsIOException(const GraphicsIOException&);
    ~GraphicsIOException() throw();
};

/** User Notify. The message should be something that can be put in
    a message box. */
class UserException : public std::runtime_error {
 public:
    UserException(const string_t& awhat);
    UserException(const char* awhat);
    UserException(const UserException&);
    ~UserException() throw();
};

/** Initialisation exception. This one is thrown when initialisation
    of some hardware fails. */
class InitException : public std::runtime_error {
 public:
    InitException(const string_t& awhat);
    InitException(const char* awhat);
    InitException(const InitException&);
    ~InitException() throw();
};

string_t perrorStr(const string_t& name);

#endif
