/**
  *  \file misc.h
  *  \brief Miscellaneous utilities
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_MISC_H
#define PCC_V2_MISC_H

#include <algorithm>
#include "cpluslib/string.h"
#include "cpluslib/types.h"
#include "defs.h"

bool        stringMatch(const char* pattern, const char* tester);
inline bool stringMatch(const char* pattern, string_t tester) { return stringMatch(pattern, tester.c_str()); }

string_t stringTokenize(string_t& line);

bool stringValue(const string_t& s, long& val);
bool stringValue(const string_t& s, long& val, string_t::size_type& pos);
bool stringValue(const string_t& s, int& val);
bool stringValue(const string_t& s, int& val, string_t::size_type& pos);

bool parseFlags(const string_t& s, const char* data, int* value, int* races, bool accept_bad_flags = false);

string_t  numToString(int32 number);
string_t  clansToString(int32 number);

#ifndef HAVE_COPY_N
/* This one is a SGI extension. */
template <class InputIterator, class Size, class OutputIterator>
inline void
copy_n_internal(InputIterator first, Size count, OutputIterator result,
                std::input_iterator_tag)
{
    while (count > 0) {
        *result = *first;
        ++first;
        ++result;
        --count;
    }
}

template <class InputIterator, class Size, class OutputIterator>
inline void
copy_n_internal(InputIterator first, Size count, OutputIterator result,
                std::random_access_iterator_tag)
{
    std::copy(first, first + count, result);
}

/* According to the manpage, it returns OutputIterator. In reality it
   returns pair<InputIterator,OutputIterator>. So we can't use the
   result anyway :-/ */
template <class InputIterator, class Size, class OutputIterator>
inline void
copy_n(InputIterator first, Size count, OutputIterator result)
{
    copy_n_internal(first, count, result,
                    std::iterator_traits<InputIterator>::iterator_category());
}
#else
using std::copy_n;
#endif

#endif
