/**
  *  \file util/recode.h
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UTIL_RECODE_H
#define PCC_V2_UTIL_RECODE_H

#include <cstddef>
#include "cpluslib/string.h"
#include "defs.h"
#include "cpluslib/types.h"

class Recoder;
class CharacterSet {
 public:
    class Data;
 private:
    const Data* data;
    friend class Recoder;
 public:
    CharacterSet(const Data* data = 0) : data(data) { }

    const char* getName() const;
    const char* getDescription() const;
    uint16      getUnicode(char c) const;
    char        getCharacter(uint16 uni) const;
    bool        isValid() const { return data; }

    bool operator==(CharacterSet rhs) const { return data==rhs.data; }
    bool operator!=(CharacterSet rhs) const { return data!=rhs.data; }
};

class Recoder {
    static const unsigned int TABLE_SIZE = 256;
    char table[TABLE_SIZE];
 public:
    Recoder(CharacterSet from, CharacterSet to);
    Recoder();

    void setTransformation(CharacterSet from, CharacterSet to);
    void setIdentityTransformation();

    string_t recodeString(string_t str) const;
    void     recodeArray(char* data, std::size_t size) const;
    void     recodeCString(char* data) const;
    /** Get recoded value of character. */
    char     recodeCharacter(char c) const { return table[(unsigned char) c]; }

    /* Enquiry */
    static CharacterSet  getCharsetByName(const string_t& name);
    static std::size_t   getNumCharsets();
    static CharacterSet  getCharset(std::size_t n);
    
 private:
    /* Static instances */
    static Recoder       intern_to_extern;
    static Recoder       extern_to_intern;
    static CharacterSet  intern_cs;
    static CharacterSet  extern_cs;

 public:
    /** Get recoder to turn internal representation into external. */
    static const Recoder& getI2E() { return intern_to_extern; }
    /** Get recoder to turn external representation into internal. */
    static const Recoder& getE2I() { return extern_to_intern; }
    /** Get handle to internally-used character set. */
    static CharacterSet getInternCS() { return intern_cs; }
    /** Get handle to externally-used character set. */
    static CharacterSet getExternCS() { return extern_cs; }
    /** Check whether character is reliable. A character is reliable if
        it survives saving and reloading. */
    static bool isReliableChar(char c) { return extern_to_intern.recodeCharacter(intern_to_extern.recodeCharacter(c)) == c; }
    
    static void setCharsets(CharacterSet intrn, const CharacterSet extrn);
};

#endif
